@@ -29,7 +29,6 @@
 #include "csoundCore.h"
 #include "pstream.h"
 #include "pvfileio.h"
-#include "oload.h"
 
 #ifdef _DEBUG
 #include <assert.h>
@@ -42,9 +41,7 @@ int fsigs_equal(const PVSDAT *f1, const PVSDAT *f2)
         && (f1->wintype == f2->wintype) /* harsh, maybe... */
         && (f1->N       == f2->N)
         && (f1->format  == f2->format)
-#ifdef SDFT
         && (f1->sliding == f2->sliding)
-#endif
         )
       return 1;
     return 0;
@@ -54,28 +51,66 @@ int fsigs_equal(const PVSDAT *f1, const PVSDAT *f2)
 /* Pandora's box opcode, but we can make it at least plausible,
    by forbidding copy to different format. */
 
+int fassign_set(CSOUND *csound, FASSIGN *p)
+{
+     int32 N = p->fsrc->N;
+
+    p->fout->N =  N;
+    p->fout->overlap = p->fsrc->overlap;
+    p->fout->winsize = p->fsrc->winsize;
+    p->fout->wintype = p->fsrc->wintype;
+    p->fout->format = p->fsrc->format;
+    p->fout->sliding = p->fsrc->sliding;
+    /* sliding needs to be checked */
+    if (p->fsrc->sliding) {
+      p->fout->NB = p->fsrc->NB;
+      csound->AuxAlloc(csound, (N + 2) * sizeof(MYFLT) * csound->ksmps,
+                       &p->fout->frame);
+      return OK;
+    }
+    if (p->fsrc->format < 0){
+      p->fout->frame.auxp = p->fsrc->frame.auxp;
+      p->fout->frame.size = p->fsrc->frame.size;
+      //csound->Message(csound, Str("fsig = : init\n"));
+    }
+    else
+      csound->AuxAlloc(csound, (N + 2) * sizeof(float), &p->fout->frame);
+    p->fout->framecount = 1;
+    //csound->Message(csound, Str("fsig = : init\n"));
+    return OK;
+}
+
+
 int fassign(CSOUND *csound, FASSIGN *p)
 {
-    int i;
-    long framesize;
+    int32 framesize;
     float *fout,*fsrc;
-    if (!fsigs_equal(p->fout,p->fsrc))
-      csound->Die(csound, Str("fsig = : formats are different.\n"));
-#ifdef SDFT
+    fout = (float *) p->fout->frame.auxp;
+    fsrc = (float *) p->fsrc->frame.auxp;
+    if (fout == fsrc) {
+      //csound->Message(csound, Str("fsig = : no cpy\n"));
+      return OK;
+    }
+    // if (UNLIKELY(!fsigs_equal(p->fout,p->fsrc)))
+    //return csound->PerfError(csound,p->h.insdshead,
+    //                         Str("fsig = : formats are different.\n"));
     if (p->fsrc->sliding) {
       memcpy(p->fout->frame.auxp, p->fsrc->frame.auxp,
              sizeof(MYFLT)*(p->fsrc->N+2)*csound->ksmps);
       return OK;
     }
-#endif
-    fout = (float *) p->fout->frame.auxp;
-    fsrc = (float *) p->fsrc->frame.auxp;
+
+
+
 
     framesize = p->fsrc->N + 2;
-    if (p->fout->framecount == p->fsrc->framecount) /* avoid duplicate copying*/
-      for (i=0;i < framesize;i++)
-        fout[i] = fsrc[i];
-    return OK;
+
+    if (p->fout->framecount == p->fsrc->framecount) {/* avoid duplicate copying*/
+    memcpy(fout, fsrc, framesize*sizeof(float));
+    p->fout->framecount++;
+    }
+
+     return OK;
 }
 
 /************* OSCBANK SYNTH ***********/
@@ -85,44 +120,46 @@ int pvadsynset(CSOUND *csound, PVADS *p)
     /* get params from input fsig */
     /* we trust they are legit! */
     int i;
-    long N = p->fsig->N;
-    long noscs,n_oscs;
-    long startbin,binoffset;
+    PVSDAT  *fs = p->fsig;
+    int32 N = fs->N;
+    int32 noscs,n_oscs;
+    int32 startbin,binoffset;
     MYFLT *p_x;
 
-#ifdef SDFT
-    if (p->fsig->sliding)
-      csound->InitError(csound, "Sliding version not yet available");
-#endif
-    p->overlap = p->fsig->overlap;
+    if (UNLIKELY(fs->sliding))
+      return csound->InitError(csound, Str("Sliding version not yet available"));
+    p->overlap = fs->overlap;
     /* a moot question, whether window params are relevant for adsyn?*/
     /* but for now, we validate all params in fsig */
-    p->winsize = p->fsig->winsize;
-    p->wintype = p->fsig->wintype;
+    p->winsize = fs->winsize;
+    p->wintype = fs->wintype;
     p->fftsize = N;
     noscs   = N/2 + 1;                     /* max possible */
-    n_oscs = (long) *p->n_oscs;    /* user param*/
-    if (n_oscs <=0)
-      csound->Die(csound, Str("pvadsyn: bad value for inoscs\n"));
+    n_oscs = (int32) *p->n_oscs;    /* user param*/
+    if (UNLIKELY(n_oscs <=0))
+      csound->InitError(csound, Str("pvadsyn: bad value for inoscs\n"));
     /* remove this when I know how to do it... */
-    if (p->fsig->format != PVS_AMP_FREQ)
-      csound->Die(csound, Str("pvadsyn: format must be amp-freq (0).\n"));
-    p->format = p->fsig->format;
+    if (UNLIKELY(fs->format != PVS_AMP_FREQ))
+      return csound->InitError(csound,
+                               Str("pvadsyn: format must be amp-freq (0).\n"));
+    p->format = fs->format;
     /* interesting question: could noscs be krate variable?
      * answer: yes, but adds complexity to oscbank,
                as have to set/reset each osc when started and stopped */
 
     /* check bin params */
-    startbin = (long) *p->ibin;            /* default 0 */
-    binoffset = (long) *p->ibinoffset; /* default 1 */
-    if (startbin        < 0 || startbin > noscs)
-      csound->Die(csound, Str("pvsadsyn: ibin parameter out of range.\n"));
-    if (startbin + n_oscs > noscs)
-      csound->Die(csound, Str("pvsadsyn: ibin + inoscs too large.\n"));
+    startbin = (int32) *p->ibin;            /* default 0 */
+    binoffset = (int32) *p->ibinoffset; /* default 1 */
+    if (UNLIKELY(startbin < 0 || startbin > noscs))
+      return csound->InitError(csound,
+                               Str("pvsadsyn: ibin parameter out of range.\n"));
+    if (UNLIKELY(startbin + n_oscs > noscs))
+      return csound->InitError(csound,
+                               Str("pvsadsyn: ibin + inoscs too large.\n"));
     /* calc final max bin target */
     p->maxosc = startbin + (n_oscs * binoffset);
-    if (p->maxosc > noscs)
-      csound->Die(csound, Str("pvsadsyn: "
+    if (UNLIKELY(p->maxosc > noscs))
+      return csound->InitError(csound, Str("pvsadsyn: "
                               "ibin + (inoscs * ibinoffset) too large."));
 
     p->outptr = 0;
@@ -142,7 +179,7 @@ int pvadsynset(CSOUND *csound, PVADS *p)
     /* initialize oscbank */
     p_x = (MYFLT *) p->x.auxp;
     for (i=0;i < noscs;i++)
-      *p_x++ = FL(1.0);
+      p_x[i] = FL(1.0);
 
     return OK;
 }
@@ -160,13 +197,13 @@ static inline MYFLT fastoscil(MYFLT *a, MYFLT *x, MYFLT *y)
 static void adsyn_frame(CSOUND *csound, PVADS *p)
 {
     int i,j;
-    long startbin,lastbin,binoffset;
+    int32 startbin,lastbin,binoffset;
     MYFLT *outbuf = (MYFLT *) (p->outbuf.auxp);
 
     float *frame;        /* RWD MUST be 32bit */
     MYFLT *a,*x,*y;
     MYFLT *amps,*freqs,*lastamps;
-    MYFLT ffac = *p->kfmod;
+    MYFLT ffac    = *p->kfmod;
     MYFLT nyquist = csound->esr * FL(0.5);
     /* we add to outbuf, so clear it first*/
     memset(p->outbuf.auxp,0,p->overlap * sizeof(MYFLT));
@@ -178,19 +215,19 @@ static void adsyn_frame(CSOUND *csound, PVADS *p)
     amps      = (MYFLT *) p->amps.auxp;
     freqs     = (MYFLT *) p->freqs.auxp;
     lastamps  = (MYFLT *) p->lastamps.auxp;
-    startbin  = (long)    *p->ibin;
-    binoffset = (long)    *p->ibinoffset;
+    startbin  = (int32)  *p->ibin;
+    binoffset = (int32)  *p->ibinoffset;
     lastbin   = p->maxosc;
 
     /*update amps, freqs*/
     for (i=startbin;i < lastbin;i+= binoffset) {
       amps[i] = frame[i*2];
       /* lazy: force all freqs positive! */
-      freqs[i] = ffac * (MYFLT) (fabs)(frame[(i*2)+1]);
+      freqs[i] = ffac * FABS(frame[(i*2)+1]);
       /* kill stuff over Nyquist. Need to worry about vlf values? */
       if (freqs[i] > nyquist)
         amps[i] = FL(0.0);
-      a[i] = (MYFLT)(2.0 * sin(freqs[i] * (double)csound->pidsr));
+      a[i] = FL(2.0) * SIN(freqs[i] * csound->pidsr);
     }
 
     /* we need to interp amplitude, but seems we can avoid doing freqs too,
@@ -225,14 +262,22 @@ static MYFLT adsyn_tick(CSOUND *csound, PVADS *p)
 
 int pvadsyn(CSOUND *csound, PVADS *p)
 {
-    int i;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
 
     MYFLT *aout = p->aout;
 
-    if (p->outbuf.auxp==NULL) {
-      csound->Die(csound, Str("pvsynth: Not initialised.\n"));
+    if (UNLIKELY(p->outbuf.auxp==NULL)) {
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsynth: Not initialised.\n"));
     }
-    for (i=0;i < csound->ksmps;i++)
+    if (UNLIKELY(offset)) memset(aout, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&aout[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset;i < nsmps;i++)
       aout[i] = adsyn_tick(csound, p);
     return OK;
 }
@@ -241,63 +286,68 @@ int pvadsyn(CSOUND *csound, PVADS *p)
 
 int pvscrosset(CSOUND *csound, PVSCROSS *p)
 {
-    long N = p->fsrc->N;
+    PVSDAT  *fsrc = p->fsrc;
+    PVSDAT  *fout = p->fout;
+    int32 N = fsrc->N;
     /* source fsig */
-    p->overlap = p->fsrc->overlap;
-    p->winsize = p->fsrc->winsize;
-    p->wintype = p->fsrc->wintype;
+    p->overlap = fsrc->overlap;
+    p->winsize = fsrc->winsize;
+    p->wintype = fsrc->wintype;
     p->fftsize = N;
-    p->format  = p->fsrc->format;
+    p->format  = fsrc->format;
 
     /* make sure fdest is same format */
-    if (!fsigs_equal(p->fsrc,p->fdest))
-      csound->Die(csound, Str("pvscross: source and dest signals "
+    if (UNLIKELY(!fsigs_equal(fsrc,p->fdest)))
+      return csound->InitError(csound, Str("pvscross: source and dest signals "
                               "must have same format\n"));
-    p->fout->N =  N;
-    p->fout->overlap = p->overlap;
-    p->fout->winsize = p->winsize;
-    p->fout->wintype = p->wintype;
-    p->fout->format = p->format;
-#ifdef SDFT
-    p->fout->format = p->fsrc->sliding;
-    if (p->fsrc->sliding) {
-      p->fout->NB = p->fsrc->NB;
-      csound->AuxAlloc(csound, (N + 2) * sizeof(MYFLT) * csound->ksmps,
-                       &p->fout->frame);
+    fout->N =  N;
+    fout->overlap = p->overlap;
+    fout->winsize = p->winsize;
+    fout->wintype = p->wintype;
+    fout->format = p->format;
+    //   fout->format = p->fsrc->sliding;
+    if (fsrc->sliding) {
+      fout->NB = fsrc->NB;
+      csound->AuxAlloc(csound, (N + 2) * sizeof(MYFLT) * CS_KSMPS,
+                       &fout->frame);
       return OK;
     }
-#endif
     /* setup output signal */
     /* RWD MUST be 32bit */
-    csound->AuxAlloc(csound, (N + 2) * sizeof(float), &p->fout->frame);
-    p->fout->framecount = 1;
+    csound->AuxAlloc(csound, (N + 2) * sizeof(float), &fout->frame);
+    fout->framecount = 1;
     p->lastframe = 0;
     return OK;
 }
 
 int pvscross(CSOUND *csound, PVSCROSS *p)
 {
-    long i,N = p->fftsize;
-    MYFLT amp1 = (MYFLT) fabs(*p->kamp1);
-    MYFLT amp2 = (MYFLT) fabs(*p->kamp2);
+    int32 i,N = p->fftsize;
+    MYFLT amp1 = FABS(*p->kamp1);
+    MYFLT amp2 = FABS(*p->kamp2);
     float *fsrc = (float *) p->fsrc->frame.auxp;    /* RWD all must be 32bit */
     float *fdest = (float *) p->fdest->frame.auxp;
     float *fout = (float *) p->fout->frame.auxp;
 
-    if (fout==NULL)
-      csound->Die(csound, Str("pvscross: not initialised\n"));
+    if (UNLIKELY(fout==NULL))
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvscross: not initialised\n"));
 
     /* make sure sigs have same format as output */
-    if (!fsigs_equal(p->fout,p->fsrc))
-      csound->Die(csound, Str("pvscross: mismatch in fsrc format\n"));
-    if (!fsigs_equal(p->fout,p->fdest))
-      csound->Die(csound, Str("pvscross: mismatch in fdest format\n"));
-#ifdef SDFT
+    if (UNLIKELY(!fsigs_equal(p->fout,p->fsrc)))
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvscross: mismatch in fsrc format\n"));
+    if (UNLIKELY(!fsigs_equal(p->fout,p->fdest)))
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvscross: mismatch in fdest format\n"));
     if (p->fsrc->sliding) {
       CMPLX *fout, *fsrc, *fdest;
-      int n, nsmps = csound->ksmps;
+      uint32_t offset = p->h.insdshead->ksmps_offset;
+      uint32_t early  = p->h.insdshead->ksmps_no_end;
+      uint32_t n, nsmps = CS_KSMPS;
       int NB = p->fsrc->NB;
-      for (n=0; n<nsmps; n++) {
+      nsmps -= early;
+      for (n=offset; n<nsmps; n++) {
         fsrc = (CMPLX *) p->fsrc->frame.auxp +n*NB;    /* RWD all must be 32bit */
         fdest = (CMPLX *) p->fdest->frame.auxp +n*NB;
         fout = (CMPLX *) p->fout->frame.auxp +n*NB;
@@ -309,7 +359,6 @@ int pvscross(CSOUND *csound, PVSCROSS *p)
       }
       return OK;
    }
-#endif
     /* only process when a new frame is ready */
     if (p->lastframe < p->fsrc->framecount) {
 #ifdef _DEBUG
@@ -327,14 +376,18 @@ int pvscross(CSOUND *csound, PVSCROSS *p)
 
 /******** PVSFREAD ************/
 
-int pvsfreadset(CSOUND *csound, PVSFREAD *p)
+static int pvsfreadset_(CSOUND *csound, PVSFREAD *p, int stringname)
 {
     PVOCEX_MEMFILE  pp;
-    unsigned long   N;
+    uint32   N;
     char            pvfilnam[MAXNAME];
 
-    csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.", p->XSTRCODE);
-    if (PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0) {
+    if (stringname) strncpy(pvfilnam, ((STRINGDAT*)p->ifilno)->data, MAXNAME-1);
+    else if (csound->ISSTRCOD(*p->ifilno))
+      strncpy(pvfilnam, get_arg_string(csound, *p->ifilno), MAXNAME-1);
+    else csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.", 0);
+
+    if (UNLIKELY(PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0)) {
       return csound->InitError(csound, Str("Failed to load PVOC-EX file"));
     }
     p->ptr = 0;
@@ -348,34 +401,33 @@ int pvsfreadset(CSOUND *csound, PVSFREAD *p)
     p->arate   = csound->esr / (MYFLT) pp.overlap;
     p->membase = (float*) pp.data;
 
-#ifdef SDFT
-    if (p->overlap < csound->ksmps || p->overlap < 10)
-      csound->InitError(csound, "Sliding version not yet available");
-#endif
-    if (p->nframes <= 0)
-      csound->Die(csound, Str("pvsfread: file is empty!\n"));
+    if (UNLIKELY(p->overlap < (int)CS_KSMPS || p->overlap < 10))
+      csound->InitError(csound, Str("Sliding version not yet available"));
+    if (UNLIKELY(p->nframes <= 0))
+      return csound->InitError(csound, Str("pvsfread: file is empty!\n"));
     /* special case if only one frame - it is an impulse response */
-    if (p->nframes == 1)
-      csound->Die(csound, Str("pvsfread: file has only one frame "
+    if (UNLIKELY(p->nframes == 1))
+      return csound->InitError(csound, Str("pvsfread: file has only one frame "
                               "(= impulse response).\n"));
-    if (p->overlap < csound->ksmps)
-      csound->Die(csound, Str("pvsfread: analysis frame overlap "
+    if (UNLIKELY(p->overlap < (int)CS_KSMPS))
+      return csound->InitError(csound, Str("pvsfread: analysis frame overlap "
                               "must be >= ksmps\n"));
     p->blockalign = (p->fftsize+2) * p->chans;
-    if ((*p->ichan) >= p->chans)
-      csound->Die(csound, Str("pvsfread: ichan value exceeds "
+    if (UNLIKELY((*p->ichan) >= p->chans))
+      return csound->InitError(csound, Str("pvsfread: ichan value exceeds "
                               "file channel count.\n"));
-    if ((long) (*p->ichan) < 0)
-      csound->Die(csound, Str("pvsfread: ichan cannot be negative.\n"));
+    if (UNLIKELY((int32) (*p->ichan) < 0))
+      return csound->InitError(csound,
+                               Str("pvsfread: ichan cannot be negative.\n"));
 
     N = p->fftsize;
     /* setup output signal */
     csound->AuxAlloc(csound, (N + 2) * sizeof(float), &p->fout->frame);
     /* init sig with first frame from file,
        regardless (always zero amps, but with bin freqs) */
-    p->chanoffset = (long) MYFLT2LRND(*p->ichan) * (N + 2);
+    p->chanoffset = (int32) MYFLT2LRND(*p->ichan) * (N + 2);
     memcpy((float *) p->fout->frame.auxp,               /* RWD MUST be 32bit */
-           (float *) pp.data + (long) p->chanoffset,    /* RWD MUST be 32bit */
+           (float *) pp.data + (int32) p->chanoffset,    /* RWD MUST be 32bit */
            (size_t) ((int) (N + 2) * (int) sizeof(float)));
     p->membase += p->blockalign;  /* move to 2nd frame in file, as startpoint */
     p->nframes--;
@@ -389,48 +441,60 @@ int pvsfreadset(CSOUND *csound, PVSFREAD *p)
     return OK;
 }
 
+
+int pvsfreadset(CSOUND *csound, PVSFREAD *p){
+  return pvsfreadset_(csound,p, 0);
+  }
+
+int pvsfreadset_S(CSOUND *csound, PVSFREAD *p){
+  return pvsfreadset_(csound,p, 1);
+  }
+
+
 int pvsfread(CSOUND *csound, PVSFREAD *p)
 {
     int i,j;
     MYFLT pos = *p->kpos;
     MYFLT framepos,frac;
-    long frame1pos,frame2pos,framesize,n_mcframes;
+    int32 frame1pos,/*frame2pos,*/framesize,n_mcframes;
 
     float *pmem,*pframe1,*pframe2;               /* RWD all MUST be 32bit */
     float *fout = (float *) p->fout->frame.auxp;
 
-    if (fout==NULL)
-      csound->Die(csound, Str("pvsfread: not initialised.\n"));
+    if (UNLIKELY(fout==NULL))
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsfread: not initialised.\n"));
 
     pmem = (float *) p->membase;
     framesize = p->fftsize+2;
     n_mcframes = p->nframes / p->chans;
 
     if (p->ptr>=p->overlap) {
-      if (pos < 0.0f)
-        pos = 0.0f;     /* or report as error... */
+      if (UNLIKELY(pos < FL(0.0)))
+        pos = FL(0.0);     /* or report as error... */
       framepos = pos * p->arate;
-      frame1pos = (long) framepos;
+      frame1pos = (int32) framepos;
 
       if (frame1pos>= n_mcframes -1) {
         /* just return final frame */
         pframe1 = pmem + (p->blockalign * (n_mcframes -1)) + p->chanoffset;
-        for (i=0;i < framesize;i++)
-          *fout++ = *pframe1++;
+        /* for (i=0;i < framesize;i++) */
+        /*   *fout++ = *pframe1++; */
+        memcpy(fout, pframe1, sizeof(float)*framesize);
       }
       else {
         /* gotta interpolate */
         /* any optimizations possible here ?
            Avoid v samll frac values ? higher-order interp ? */
-        frame2pos = frame1pos+p->chans;
+        //frame2pos = frame1pos+p->chans;
         frac = framepos - (MYFLT) frame1pos;
         pframe1 = pmem + (frame1pos * p->blockalign) + p->chanoffset;
         pframe2 = pframe1 + p->blockalign;
         for (i=0;i < framesize;i+=2) {
           MYFLT amp,freq;
-          j = i+1;
-          amp = pframe1[i] + frac * (pframe2[i] - pframe1[i]);
-          freq = pframe1[j] + frac * (pframe2[j] - pframe1[j]);
+          j       = i+1;
+          amp     = pframe1[i] + frac * (pframe2[i] - pframe1[i]);
+          freq    = pframe1[j] + frac * (pframe2[j] - pframe1[j]);
           fout[i] = (float) amp;
           fout[j] = (float) freq;
         }
@@ -439,7 +503,7 @@ int pvsfread(CSOUND *csound, PVSFREAD *p)
       p->fout->framecount++;
       p->lastframe = p->fout->framecount;
     }
-    p->ptr += csound->ksmps;
+    p->ptr += CS_KSMPS;
 
     return OK;
 }
@@ -447,18 +511,18 @@ int pvsfread(CSOUND *csound, PVSFREAD *p)
 /************* PVSMASKA ****************/
 int pvsmaskaset(CSOUND *csound, PVSMASKA *p)
 {
-    int i;
+    unsigned int i;
     MYFLT *ftable;
-    long N = p->fsrc->N;
-    long nbins = N/2 + 1;
+    uint32_t N = p->fsrc->N;
+    uint32_t nbins = N/2 + 1;
     /* source fsig */
     p->overlap = p->fsrc->overlap;
     p->winsize = p->fsrc->winsize;
     p->wintype = p->fsrc->wintype;
     p->format  = p->fsrc->format;
     p->fftsize = N;
-    if (!(p->format==PVS_AMP_FREQ) || (p->format==PVS_AMP_PHASE))
-      csound->Die(csound, Str("pvsmaska: "
+    if (UNLIKELY(!(p->format==PVS_AMP_FREQ) || (p->format==PVS_AMP_PHASE)))
+      return csound->InitError(csound, Str("pvsmaska: "
                               "signal format must be amp-phase or amp-freq."));
     /* setup output signal */
     p->fout->N =  N;
@@ -466,28 +530,26 @@ int pvsmaskaset(CSOUND *csound, PVSMASKA *p)
     p->fout->winsize = p->winsize;
     p->fout->wintype = p->wintype;
     p->fout->format  = p->format;
-#ifdef SDFT
     p->fout->sliding = p->fsrc->sliding;
     if (p->fsrc->sliding) {
-      csound->AuxAlloc(csound, (N + 2) * sizeof(MYFLT) * csound->ksmps,
+      csound->AuxAlloc(csound, (N + 2) * sizeof(MYFLT) * CS_KSMPS,
                        &p->fout->frame);
       p->fout->NB = p->fsrc->NB;
     }
     else
-#endif
       {
         /* RWD MUST be 32bit */
         csound->AuxAlloc(csound, (N + 2) * sizeof(float), &p->fout->frame);
         p->fout->framecount = 1;
         p->lastframe = 0;
       }
-    p->maskfunc = csound->FTFind(csound, p->ifn);
-    if (p->maskfunc==NULL)
+    p->maskfunc = csound->FTnp2Find(csound, p->ifn);
+    if (UNLIKELY(p->maskfunc==NULL))
       return NOTOK;
 
     /* require at least size of nbins */
-    if (p->maskfunc->flen + 1 <         nbins)
-      csound->Die(csound, Str("pvsmaska: ftable too small.\n"));
+    if (UNLIKELY(p->maskfunc->flen + 1 <         nbins))
+      return csound->InitError(csound, Str("pvsmaska: ftable too small.\n"));
 
     /* clip any negative values in table */
     ftable = p->maskfunc->ftable;
@@ -501,52 +563,53 @@ int pvsmaskaset(CSOUND *csound, PVSMASKA *p)
 int pvsmaska(CSOUND *csound, PVSMASKA *p)
 {
     int i;
-    long flen, nbins;
+    int32 /*flen, */nbins;
     MYFLT *ftable;
     float *fout,*fsrc;                      /* RWD MUST be 32bit */
-    float margin,depth = (float)*p->kdepth;
+    float margin, depth = (float)*p->kdepth;
 
-    flen = p->maskfunc->flen + 1;
+    //flen = p->maskfunc->flen + 1;
     ftable = p->maskfunc->ftable;
     fout = (float *) p->fout->frame.auxp;   /* RWD both MUST be 32bit */
     fsrc = (float *) p->fsrc->frame.auxp;
 
-    if (fout==NULL)
-      csound->Die(csound, Str("pvsmaska: not initialised\n"));
+    if (UNLIKELY(fout==NULL))
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsmaska: not initialised\n"));
 
     if (depth < FL(0.0)) {
       /* need the warning: but krate linseg can give below-zeroes incorrectly */
-      if (!p->nwarned)  {
-        if (csound->oparms->msglevel & WARNMSG)
-          csound->Message(csound, Str("WARNING: pvsmaska: negative value for kdepth; "
+      if (UNLIKELY(!p->nwarned))  {
+        csound->Warning(csound, Str("pvsmaska: negative value for kdepth; "
                      "clipped to zero.\n"));
         p->nwarned = 1;
       }
-      depth = FL(0.0);
+      depth = 0.0f;
     }
-    if (depth > 1.0f) {
-      if (!p->pwarned)  {
-        if (csound->oparms->msglevel & WARNMSG)
-          csound->Message(csound, Str("WARNING: pvsmaska: kdepth > 1: clipped.\n"));
+    if (UNLIKELY(depth > 1.0f)) {
+      if (UNLIKELY(!p->pwarned))  {
+        csound->Warning(csound, Str("pvsmaska: kdepth > 1: clipped.\n"));
         p->pwarned = 1;
       }
-      depth = FL(1.0);
+      depth = 1.0f;
     }
     /* massage into internal mapping */
     /* user 0 --> my 1
        user 1 ---> my 0
        user 0.75 -->my 0.25
     */
-    depth = (float)(FL(1.0) - depth);
+    depth = (1.0f - depth);
 
-    margin = (float)(FL(1.0) - depth);
-#ifdef SDFT
+    margin = 1.0f - depth;
     if (p->fsrc->sliding) {
       int NB = p->fsrc->NB;
       CMPLX *fout, *fsrc;
-      int n, nsmps=csound->ksmps;
-      MYFLT amp = 1.0f;
-      for (n=0; n<nsmps; n++) {
+      uint32_t offset = p->h.insdshead->ksmps_offset;
+      uint32_t early  = p->h.insdshead->ksmps_no_end;
+      uint32_t n, nsmps = CS_KSMPS;
+      MYFLT amp = FL(1.0);
+      nsmps -= early;
+      for (n=offset; n<nsmps; n++) {
         fout = (CMPLX *) p->fout->frame.auxp +n*NB;
         fsrc = (CMPLX *) p->fsrc->frame.auxp +n*NB;
         for (i=0; i<NB; i++) {
@@ -557,11 +620,10 @@ int pvsmaska(CSOUND *csound, PVSMASKA *p)
       }
       return OK;
     }
-#endif
     nbins = p->fftsize/2 + 1;
     /* only process when a new frame is ready */
     if (p->lastframe < p->fsrc->framecount) {
-      MYFLT amp = 1.0f;
+      MYFLT amp = FL(1.0);
       int aindex, findex;
       for (i=0;i < nbins;i++) {
         amp = depth + margin * ftable[i];
@@ -584,37 +646,35 @@ int pvsftwset(CSOUND *csound, PVSFTW *p)
     int i;
     MYFLT *ftablea,*ftablef;
     float *fsrc;                                        /* RWD MUST be 32bit */
-    long flena,flenf,nbins, N = p->fsrc->N;
+    int32 flena,flenf,nbins, N = p->fsrc->N;
     /* source fsig */
     p->overlap = p->fsrc->overlap;
     p->winsize = p->fsrc->winsize;
     p->wintype = p->fsrc->wintype;
     p->format  = p->fsrc->format;
     p->fftsize = N;
-    p->outfna = p->outfnf = NULL;
+    p->outfna  = p->outfnf = NULL;
     p->lastframe = 0;
 
-    if (!(p->format==PVS_AMP_FREQ) || (p->format==PVS_AMP_PHASE))
-      csound->Die(csound, Str("pvsftw: signal format must be "
+    if (UNLIKELY(!(p->format==PVS_AMP_FREQ) || (p->format==PVS_AMP_PHASE)))
+      return csound->InitError(csound, Str("pvsftw: signal format must be "
                               "amp-phase or amp-freq.\n"));
-    if (*p->ifna < 1.0f)
-      csound->Die(csound, Str("pvsftw: bad value for ifna.\n"));
-    if (*p->ifnf < 0.0f)                /* 0 = notused */
-      csound->Die(csound, Str("pvsftw: bad value for ifnf.\n"));
-    p->outfna = csound->FTFind(csound, p->ifna);
-    if (p->outfna==NULL)
+    if (UNLIKELY(*p->ifna < 1.0f))
+      return csound->InitError(csound, Str("pvsftw: bad value for ifna.\n"));
+    if (UNLIKELY(*p->ifnf < 0.0f))                /* 0 = notused */
+      return csound->InitError(csound, Str("pvsftw: bad value for ifnf.\n"));
+    p->outfna = csound->FTnp2Find(csound, p->ifna);
+    if (UNLIKELY(p->outfna==NULL))
       return NOTOK;
-#ifdef SDFT
-    if (p->fsrc->sliding)
-      csound->InitError(csound, "Sliding version not yet available");
-#endif
+    if (UNLIKELY(p->fsrc->sliding))
+      csound->InitError(csound, Str("Sliding version not yet available"));
     fsrc = (float *) p->fsrc->frame.auxp;               /* RWD MUST be 32bit */
     /* init table, one presumes with zero amps */
     nbins = p->fftsize/2 + 1;
     flena = p->outfna->flen + 1;
 
-    if (flena < nbins)
-      csound->Die(csound, Str("pvsftw: amps ftable too small.\n"));
+    if (UNLIKELY(flena < nbins))
+      return csound->InitError(csound, Str("pvsftw: amps ftable too small.\n"));
 
     /* init tables */
     ftablea = p->outfna->ftable;
@@ -622,15 +682,16 @@ int pvsftwset(CSOUND *csound, PVSFTW *p)
       ftablea[i] = fsrc[i*2];
 
     /* freq table? */
-    if ((long) *p->ifnf >= 1.0f) {
-      p->outfnf = csound->FTFind(csound, p->ifnf);
-      if (p->outfnf==NULL)
+    if ((int32) *p->ifnf >= 1) {
+      p->outfnf = csound->FTnp2Find(csound, p->ifnf);
+      if (UNLIKELY(p->outfnf==NULL))
         return NOTOK;
       ftablef = p->outfnf->ftable;
       if (ftablef) {
         flenf = p->outfnf->flen+1;
-        if (flenf < nbins)
-          csound->Die(csound, Str("pvsftw: freqs ftable too small.\n"));
+        if (UNLIKELY(flenf < nbins))
+          return csound->InitError(csound,
+                                   Str("pvsftw: freqs ftable too small.\n"));
         for (i=0;i < nbins;i++)
           ftablef[i] = fsrc[(i*2) + 1];
       }
@@ -642,21 +703,24 @@ int pvsftwset(CSOUND *csound, PVSFTW *p)
 int pvsftw(CSOUND *csound, PVSFTW *p)
 {
     int i;
-    long nbins;
+    int32 nbins;
     MYFLT *ftablea, *ftablef = NULL;
     float *fsrc;                /* RWD MUST be 32bit */
 
     ftablea = p->outfna->ftable;
     fsrc = (float *) p->fsrc->frame.auxp;
 
-    if (fsrc==NULL)
-      csound->Die(csound, Str("pvsftw: not initialised\n"));
-    if (ftablea==NULL)
-      csound->Die(csound, Str("pvsftw: no amps ftable!\n"));
+    if (UNLIKELY(fsrc==NULL))
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsftw: not initialised\n"));
+    if (UNLIKELY(ftablea==NULL))
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsftw: no amps ftable!\n"));
     if (p->outfnf) {
       ftablef = p->outfnf->ftable;
-      if (ftablef==NULL)
-        csound->Die(csound, Str("pvsftw: no freqs ftable!\n"));
+      if (UNLIKELY(ftablef==NULL))
+        return csound->PerfError(csound,p->h.insdshead,
+                                 Str("pvsftw: no freqs ftable!\n"));
     }
     nbins = p->fftsize/2 + 1;
 
@@ -685,43 +749,41 @@ int pvsftrset(CSOUND *csound, PVSFTR *p)
 {
     int i;
     float *fdest;                         /* RWD MUST be 32bit */
-    long flena,flenf,nbins, N = p->fdest->N;
+    int32 flena,flenf,nbins, N = p->fdest->N;
     /* source fsig */
-    p->overlap = p->fdest->overlap;
-    p->winsize = p->fdest->winsize;
-    p->wintype = p->fdest->wintype;
-    p->format  = p->fdest->format;
-    p->fftsize = N;
-    p->infna = p->infnf = NULL;
-    p->ftablea = p->ftablef = NULL;
+    p->overlap   = p->fdest->overlap;
+    p->winsize   = p->fdest->winsize;
+    p->wintype   = p->fdest->wintype;
+    p->format    = p->fdest->format;
+    p->fftsize   = N;
+    p->infna     = p->infnf = NULL;
+    p->ftablea   = p->ftablef = NULL;
     p->lastframe = 0;
-    nbins = p->fftsize/2 + 1;
+    nbins        = p->fftsize/2 + 1;
 
-    if (!(p->format==PVS_AMP_FREQ) || (p->format==PVS_AMP_PHASE))
-      csound->Die(csound, Str("pvsftr: signal format must be "
+    if (UNLIKELY(!(p->format==PVS_AMP_FREQ) || (p->format==PVS_AMP_PHASE)))
+      return csound->InitError(csound, Str("pvsftr: signal format must be "
                               "amp-phase or amp-freq.\n"));
     /* ifn = 0 = notused */
-    if (*p->ifna < 0.0f)
-      csound->Die(csound, Str("pvsftr: bad value for ifna.\n"));
-    if (*p->ifnf < 0.0f)
-      csound->Die(csound, Str("pvsftr: bad value for ifnf.\n"));
+    if (UNLIKELY(*p->ifna < FL(0.0)))
+      return csound->InitError(csound, Str("pvsftr: bad value for ifna.\n"));
+    if (UNLIKELY(*p->ifnf < FL(0.0)))
+      return csound->InitError(csound, Str("pvsftr: bad value for ifnf.\n"));
 
     /* if ifna=0; no amps to read; one assumes the user ias changing freqs only,
        otherwise, there is no change!
     */
-    if ((long) *p->ifna != 0) {
-      p->infna = csound->FTFind(csound, p->ifna);
-      if (p->infna==NULL)
+    if ((int32) *p->ifna != 0) {
+      p->infna = csound->FTnp2Find(csound, p->ifna);
+      if (UNLIKELY(p->infna==NULL))
         return NOTOK;
       p->ftablea = p->infna->ftable;
       flena = p->infna->flen + 1;
-      if (flena < nbins)
-        csound->Die(csound, Str("pvsftr: amps ftable too small.\n"));
+      if (UNLIKELY(flena < nbins))
+        return csound->InitError(csound, Str("pvsftr: amps ftable too small.\n"));
     }
-#ifdef SDFT
-    if (p->overlap < csound->ksmps || p->overlap < 10)
-      csound->InitError(csound, "Sliding version not yet available");
-#endif
+    if (UNLIKELY(p->overlap < (int)CS_KSMPS || p->overlap < 10))
+      csound->InitError(csound, Str("Sliding version not yet available"));
     fdest = (float *) p->fdest->frame.auxp;             /* RWD MUST be 32bit */
 
     /*** setup first frame ?? */
@@ -730,14 +792,14 @@ int pvsftrset(CSOUND *csound, PVSFTR *p)
         fdest[i*2] = (float) p->ftablea[i];
 
     /* freq table? */
-    if ((long) *p->ifnf >= 1) {
-      p->infnf = csound->FTFind(csound, p->ifnf);
-      if (p->infnf==NULL)
+    if ((int32) *p->ifnf >= 1) {
+      p->infnf = csound->FTnp2Find(csound, p->ifnf);
+      if (UNLIKELY(p->infnf==NULL))
         return NOTOK;
       p->ftablef = p->infnf->ftable;
       flenf = p->infnf->flen+1;
-      if (flenf < nbins)
-        csound->Die(csound, Str("pvsftr: freqs ftable too small.\n"));
+      if (UNLIKELY(flenf < nbins))
+        return csound->InitError(csound, Str("pvsftr: freqs ftable too small.\n"));
       for (i=0;i < nbins;i++)
         fdest[(i*2) + 1] = (float) p->ftablef[i];
 
@@ -748,13 +810,14 @@ int pvsftrset(CSOUND *csound, PVSFTR *p)
 
 int pvsftr(CSOUND *csound, PVSFTR *p)
 {
-    int i;
-    long nbins;
+    int   i;
+    int32 nbins;
     float *fdest;                       /*RWD MUST be 32bit */
 
     fdest = (float *) p->fdest->frame.auxp;
-    if (fdest==NULL)
-      csound->Die(csound, Str("pvsftr: not initialised\n"));
+    if (UNLIKELY(fdest==NULL))
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsftr: not initialised\n"));
     nbins = p->fftsize/2 + 1;
 
     /* only write when a new frame is ready */
@@ -786,4 +849,3 @@ int pvsinfo(CSOUND *csound, PVSINFO *p)
     *p->iformat  = (MYFLT) p->fsrc->format;
     return OK;
 }
-
