@@ -24,19 +24,18 @@
 #include "csoundCore.h"         /*                              UGENS3.C    */
 #include "ugens3.h"
 #include <math.h>
-#include "oload.h"
 
 int foscset(CSOUND *csound, FOSC *p)
 {
     FUNC    *ftp;
 
-    if ((ftp = csound->FTFind(csound, p->ifn)) != NULL) {
+    if (LIKELY((ftp = csound->FTFind(csound, p->ifn)) != NULL)) {
       p->ftp = ftp;
       if (*p->iphs >= 0)
-        p->cphs = p->mphs = (long)(*p->iphs * FMAXLEN);
-      p->ampcod = (XINARG1) ? 1 : 0;
-      p->carcod = (XINARG3) ? 1 : 0;
-      p->modcod = (XINARG4) ? 1 : 0;
+        p->cphs = p->mphs = (int32)(*p->iphs * FMAXLEN);
+      p->ampcod = IS_ASIG_ARG(p->xamp) ? 1 : 0;
+      p->carcod = IS_ASIG_ARG(p->xcar) ? 1 : 0;
+      p->modcod = IS_ASIG_ARG(p->xmod) ? 1 : 0;
       return OK;
     }
     return NOTOK;
@@ -47,14 +46,15 @@ int foscil(CSOUND *csound, FOSC *p)
     FUNC    *ftp;
     MYFLT   *ar, *ampp, *modp, cps, amp;
     MYFLT   xcar, xmod, *carp, car, fmod, cfreq, mod, ndx, *ftab;
-    long    mphs, cphs, minc, cinc, lobits;
-    int     n;
+    int32    mphs, cphs, minc, cinc, lobits;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    MYFLT   sicvt = csound->sicvt;
 
     ar = p->rslt;
     ftp = p->ftp;
-    if (ftp == NULL) {
-      return csound->PerfError(csound, Str("foscil: not initialised"));
-    }
+    if (UNLIKELY(ftp == NULL)) goto err1;
     ftab = ftp->ftable;
     lobits = ftp->lobits;
     mphs = p->mphs;
@@ -66,21 +66,26 @@ int foscil(CSOUND *csound, FOSC *p)
     amp  = *ampp;
     xcar = *carp;
     xmod = *modp;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
 
-    if (p->XINCODE) {
-      for (n=0;n<csound->ksmps;n++) {
+    if (p->ampcod || p->carcod || p->modcod) {
+      for (n=offset;n<nsmps;n++) {
         if (p->ampcod) amp = ampp[n];
         if (p->carcod) xcar = carp[n];
         if (p->modcod) xmod = modp[n];
         car = cps * xcar;
         mod = cps * xmod;
         ndx = *p->kndx * mod;
-        minc = (long)(mod * csound->sicvt);
+        minc = (int32)(mod * sicvt);
         mphs &= PHMASK;
         fmod = *(ftab + (mphs >>lobits)) * ndx;
         mphs += minc;
         cfreq = car + fmod;
-        cinc = (long)(cfreq * csound->sicvt);
+        cinc = (int32)(cfreq * sicvt);
         cphs &= PHMASK;
         ar[n] = *(ftab + (cphs >>lobits)) * amp;
         cphs += cinc;
@@ -92,13 +97,13 @@ int foscil(CSOUND *csound, FOSC *p)
       car = cps * *carp;
       mod = cps * *modp;
       ndx = *p->kndx * mod;
-      minc = (long)(mod * csound->sicvt);
-      for (n=0;n<csound->ksmps;n++) {
+      minc = (int32)(mod * sicvt);
+      for (n=offset;n<nsmps;n++) {
         mphs &= PHMASK;
         fmod = *(ftab + (mphs >>lobits)) * ndx;
         mphs += minc;
         cfreq = car + fmod;
-        cinc = (long)(cfreq * csound->sicvt);
+        cinc = (int32)(cfreq * sicvt);
         cphs &= PHMASK;
         ar[n] = *(ftab + (cphs >>lobits)) * amp;
         cphs += cinc;
@@ -108,6 +113,9 @@ int foscil(CSOUND *csound, FOSC *p)
     p->cphs = cphs;
 
     return OK;
+ err1:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("foscil: not initialised"));
 }
 
 int foscili(CSOUND *csound, FOSC *p)
@@ -115,14 +123,17 @@ int foscili(CSOUND *csound, FOSC *p)
     FUNC   *ftp;
     MYFLT  *ar, *ampp, amp, cps, fract, v1, car, fmod, cfreq, mod;
     MYFLT  *carp, *modp, xcar, xmod, ndx, *ftab;
-    long   mphs, cphs, minc, cinc, lobits;
-    int    n;
+    int32  mphs, cphs, minc, cinc, lobits;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    MYFLT  sicvt = csound->sicvt;
+    MYFLT  *ft;
 
     ar = p->rslt;
     ftp = p->ftp;
-    if (ftp == NULL) {        /* RWD fix */
-      return csound->PerfError(csound, Str("foscili: not initialised"));
-    }
+    if (UNLIKELY(ftp == NULL)) goto err1;        /* RWD fix */
+    ft = ftp->ftable;
     lobits = ftp->lobits;
     mphs = p->mphs;
     cphs = p->cphs;
@@ -133,26 +144,31 @@ int foscili(CSOUND *csound, FOSC *p)
     amp  = *ampp;
     xcar = *carp;
     xmod = *modp;
-    if (p->XINCODE) {
-      for (n=0;n<csound->ksmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    if (p->ampcod || p->carcod || p->modcod) {
+      for (n=offset;n<nsmps;n++) {
         if (p->ampcod)  amp = ampp[n];
         if (p->carcod)  xcar = carp[n];
         if (p->modcod)  xmod = modp[n];
         car = cps * xcar;
         mod = cps * xmod;
         ndx = *p->kndx * mod;
-        minc = (long)(mod * csound->sicvt);
+        minc = (int32)(mod * sicvt);
         mphs &= PHMASK;
         fract = PFRAC(mphs);
-        ftab = ftp->ftable + (mphs >>lobits);
+        ftab = ft + (mphs >>lobits);
         v1 = *ftab++;
         fmod = (v1 + (*ftab - v1) * fract) * ndx;
         mphs += minc;
         cfreq = car + fmod;
-        cinc = (long)(cfreq * csound->sicvt);
+        cinc = (int32)(cfreq * sicvt);
         cphs &= PHMASK;
         fract = PFRAC(cphs);
-        ftab = ftp->ftable + (cphs >>lobits);
+        ftab = ft + (cphs >>lobits);
         v1 = *ftab++;
         ar[n] = (v1 + (*ftab - v1) * fract) * amp;
         cphs += cinc;
@@ -163,19 +179,19 @@ int foscili(CSOUND *csound, FOSC *p)
       car = cps * *carp;
       mod = cps * *modp;
       ndx = *p->kndx * mod;
-      minc = (long)(mod * csound->sicvt);
-      for (n=0;n<csound->ksmps;n++) {
+      minc = (int32)(mod * sicvt);
+      for (n=offset;n<nsmps;n++) {
         mphs &= PHMASK;
         fract = PFRAC(mphs);
-        ftab = ftp->ftable + (mphs >>lobits);
+        ftab = ft + (mphs >>lobits);
         v1 = *ftab++;
         fmod = (v1 + (*ftab - v1) * fract) * ndx;
         mphs += minc;
         cfreq = car + fmod;
-        cinc = (long)(cfreq * csound->sicvt);
+        cinc = (int32)(cfreq * sicvt);
         cphs &= PHMASK;
         fract = PFRAC(cphs);
-        ftab = ftp->ftable + (cphs >>lobits);
+        ftab = ft + (cphs >>lobits);
         v1 = *ftab++;
         ar[n] = (v1 + (*ftab - v1) * fract) * amp;
         cphs += cinc;
@@ -185,65 +201,73 @@ int foscili(CSOUND *csound, FOSC *p)
     p->cphs = cphs;
 
     return OK;
+ err1:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("foscili: not initialised"));
 }
 
+
 int losset(CSOUND *csound, LOSC *p)
 {
     FUNC    *ftp;
     if ((ftp = csound->FTnp2Find(csound,p->ifn)) != NULL) {
-      long  maxphs = ((long) ftp->flenfrms << LOBITS) + ((long) LOFACT - 1);
+      uint32 maxphs = ((int32) ftp->flenfrms << LOBITS) + ((int32) LOFACT - 1);
       p->ftp = ftp;
       if (*p->ibas != FL(0.0))
         p->cpscvt = ftp->cvtbas / *p->ibas;
-      else if ((p->cpscvt = ftp->cpscvt) == FL(0.0)) {
+      else if (UNLIKELY((p->cpscvt = ftp->cpscvt) == FL(0.0))) {
         p->cpscvt = FL(261.62561); /* Middle C */
-        if (csound->oparms->msglevel & WARNMSG)
-          csound->Warning(csound, Str("no legal base frequency"));
+        csound->Warning(csound, Str("no legal base frequency"));
       }
-      if ((p->mod1 = (short) *p->imod1) < 0) {
-        if ((p->mod1 = ftp->loopmode1) == 0) {
-          if (csound->oparms->msglevel & WARNMSG)
-            csound->Warning(csound, Str("loscil: sustain defers to "
-                                        "non-looping source"));
+      if ((p->mod1 = (int16) *p->imod1) < 0) {
+        if (UNLIKELY((p->mod1 = ftp->loopmode1) == 0)) {
+          csound->Warning(csound, Str("loscil: sustain defers to "
+                                      "non-looping source"));
         }
         p->beg1 = ftp->begin1 << LOBITS;
         p->end1 = ftp->end1 << LOBITS;
       }
-      else if (p->mod1 < 0 || p->mod1 > 3)
+      else if (UNLIKELY(p->mod1 < 0 || p->mod1 > 3))
         goto lerr2;
       else {
-        p->beg1 = (long) (*p->ibeg1 * (MYFLT) (LOFACT));
-        p->end1 = (long) (*p->iend1 * (MYFLT) (LOFACT));
+        p->beg1 = (int32) (*p->ibeg1 * (MYFLT) (LOFACT));
+        p->end1 = (int32) (*p->iend1 * (MYFLT) (LOFACT));
         if (!p->beg1 && !p->end1) {
           /* default to looping the whole sample */
-          p->end1 = (p->mod1 ? maxphs : ((long) ftp->flenfrms << LOBITS));
+          p->end1 = (p->mod1 ? (int32)maxphs : ((int32) ftp->flenfrms << LOBITS));
         }
-        else if (p->beg1 < 0 || p->end1 > maxphs || p->beg1 >= p->end1)
+        else if (UNLIKELY(p->beg1 < 0 ||
+                          p->end1 > (int32)maxphs ||
+                          p->beg1 >= p->end1)) {
+          csound->Message(csound, "beg: %d, end = %d, maxphs = %d\n",
+                          p->beg1, p->end1, maxphs);
           goto lerr2;
+        }
       }
-      if ((p->mod2 = (short) *p->imod2) < 0) {
+      if ((p->mod2 = (int16) *p->imod2) < 0) {
         p->mod2 = ftp->loopmode2;
         p->beg2 = ftp->begin2 << LOBITS;
         p->end2 = ftp->end2 << LOBITS;
       }
       else {
-        p->beg2 = (long) (*p->ibeg2 * (MYFLT) (LOFACT));
-        p->end2 = (long) (*p->iend2 * (MYFLT) (LOFACT));
-        if (p->mod2 < 0 || p->mod2 > 3 ||
-            p->beg2 < 0 || p->end2 > maxphs || p->beg2 >= p->end2) {
+        p->beg2 = (int32) (*p->ibeg2 * (MYFLT) (LOFACT));
+        p->end2 = (int32) (*p->iend2 * (MYFLT) (LOFACT));
+        if (UNLIKELY(p->mod2 < 0 || p->mod2 > 3 ||
+                     p->beg2 < 0 || p->end2 > (int32)maxphs ||
+                     p->beg2 >= p->end2)) {
           goto lerr3;
         }
       }
       p->beg1 = (p->beg1 >= 0L ? p->beg1 : 0L);
-      p->end1 = (p->end1 < maxphs ? p->end1 : maxphs);
-      if (p->beg1 >= p->end1) {
+      p->end1 = (p->end1 < (int32)maxphs ? p->end1 : (int32)maxphs);
+      if (UNLIKELY(p->beg1 >= p->end1)) {
         p->mod1 = 0;
         p->beg1 = 0L;
         p->end1 = maxphs;
       }
       p->beg2 = (p->beg2 >= 0L ? p->beg2 : 0L);
-      p->end2 = (p->end2 < maxphs ? p->end2 : maxphs);
-      if (p->beg2 >= p->end2) {
+      p->end2 = (p->end2 < (int32)maxphs ? p->end2 : (int32)maxphs);
+      if (UNLIKELY(p->beg2 >= p->end2)) {
         p->mod2 = 0;
         p->beg2 = 0L;
       }
@@ -256,13 +280,13 @@ int losset(CSOUND *csound, LOSC *p)
       else p->looping = 0;
       if (p->OUTOCOUNT == 1) {
         p->stereo = 0;
-        if (ftp->nchanls != 1)
+        if (UNLIKELY(ftp->nchanls != 1))
           return csound->InitError(csound, Str(
                                "mono loscil cannot read from stereo ftable"));
       }
       else {
         p->stereo = 1;
-        if (ftp->nchanls != 2)
+        if (UNLIKELY(ftp->nchanls != 2))
           return csound->InitError(csound, Str(
                                "stereo loscil cannot read from mono ftable"));
       }
@@ -277,7 +301,7 @@ int losset(CSOUND *csound, LOSC *p)
 }
 
 static inline void loscil_linear_interp_mono(MYFLT *ar,
-                                             MYFLT *ftbl, long phs, long flen)
+                                             MYFLT *ftbl, int32 phs, int32 flen)
 {
     MYFLT   fract, tmp;
     int     x;
@@ -290,7 +314,7 @@ static inline void loscil_linear_interp_mono(MYFLT *ar,
 }
 
 static inline void loscil_linear_interp_stereo(MYFLT *arL, MYFLT *arR,
-                                               MYFLT *ftbl, long phs, long flen)
+                                               MYFLT *ftbl, int32 phs, int32 flen)
 {
     MYFLT   fract, tmpL, tmpR;
     int     x;
@@ -305,7 +329,7 @@ static inline void loscil_linear_interp_stereo(MYFLT *arL, MYFLT *arR,
 }
 
 static inline void loscil_cubic_interp_mono(MYFLT *ar,
-                                            MYFLT *ftbl, long phs, long flen)
+                                            MYFLT *ftbl, int32 phs, int32 flen)
 {
     MYFLT   fract, tmp, a0, a1, a2, a3;
     int     x;
@@ -327,7 +351,7 @@ static inline void loscil_cubic_interp_mono(MYFLT *ar,
 
 static CS_NOINLINE void
     loscil_cubic_interp_stereo(MYFLT *arL, MYFLT *arR,
-                               MYFLT *ftbl, long phs, long flen)
+                               MYFLT *ftbl, int32 phs, int32 flen)
 {
     MYFLT   fract, tmpL, tmpR, a0, a1, a2, a3;
     int     x;
@@ -353,23 +377,29 @@ static CS_NOINLINE void
     *arR = tmpR;
 }
 
+/* *********************** needs total rewrite **************** */
 int loscil(CSOUND *csound, LOSC *p)
 {
     FUNC    *ftp;
     MYFLT   *ar1, *ar2, *ftbl, *xamp;
-    long    phs, inc, beg, end;
-    int     nsmps = csound->ksmps, aamp;
+    int32    phs, inc, beg, end;
+    uint32_t n = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nsmps = CS_KSMPS;
+    int      aamp;
+    MYFLT    xx;
 
     ftp = p->ftp;
     ftbl = ftp->ftable;
-    if ((inc = (long)(*p->kcps * p->cpscvt)) < 0)
+    if ((inc = (int32)(*p->kcps * p->cpscvt)) < 0)
       inc = -inc;
     xamp = p->xamp;
-    aamp = (p->XINCODE) ? 1 : 0;
+    xx = *xamp;
+    aamp = IS_ASIG_ARG(p->xamp) ? 1 : 0;
     if (p->seg1) {                      /* if still segment 1  */
       beg = p->beg1;
       end = p->end1;
-      if (p->h.insdshead->relesing)     /*    sense note_off   */
+      if (UNLIKELY(p->h.insdshead->relesing))     /*    sense note_off   */
         p->looping = 0;
     }
     else {
@@ -378,8 +408,15 @@ int loscil(CSOUND *csound, LOSC *p)
     }
     phs = p->lphs;
     ar1 = p->ar1;
+    if (UNLIKELY(n)) memset(ar1, '\0', n*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar1[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->stereo) {
       ar2 = p->ar2;
+      if (UNLIKELY(n)) memset(ar2, '\0', n*sizeof(MYFLT));
+      if (UNLIKELY(early)) memset(&ar2[nsmps], '\0', early*sizeof(MYFLT));
       goto phsck2;
     }
  phschk:
@@ -387,53 +424,53 @@ int loscil(CSOUND *csound, LOSC *p)
       goto put0;
     switch (p->curmod) {
     case 0:
-      do {                                      /* NO LOOPING  */
-        loscil_linear_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* NO LOOPING  */
+        loscil_linear_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
         if ((phs += inc) >= end)
           goto nxtseg;
-      } while (--nsmps);
+      }
       break;
     case 1:
-      do {                                      /* NORMAL LOOPING */
-        loscil_linear_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs += inc) >= end) {
+      for (; n<nsmps; n++) {                    /* NORMAL LOOPING */
+        loscil_linear_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg;
           phs -= end - beg;
         }
-      } while (--nsmps);
+      }
       break;
     case 2:
     case2:
-      do {                                      /* BIDIR FORW, EVEN */
-        loscil_linear_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* BIDIR FORW, EVEN */
+        loscil_linear_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
         if ((phs += inc) >= end) {
           if (!(p->looping)) goto nxtseg;
           phs -= (phs - end) * 2;
           p->curmod = 3;
-          if (--nsmps) goto case3;
+          if (++n<nsmps) goto case3;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
     case 3:
     case3:
-      do {                                      /* BIDIR BACK, EVEN */
-        loscil_linear_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs -= inc) < beg) {
+      for (; n<nsmps; n++) {                    /* BIDIR BACK, EVEN */
+        loscil_linear_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        if (UNLIKELY((phs -= inc) < beg)) {
           phs += (beg - phs) * 2;
           p->curmod = 2;
-          if (--nsmps) goto case2;
+          if (++n<nsmps) goto case2;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
 
     nxtseg:
@@ -441,7 +478,7 @@ int loscil(CSOUND *csound, LOSC *p)
         p->seg1 = 0;
         if ((p->curmod = p->mod2) != 0)
           p->looping = 1;
-        if (--nsmps) {
+        if (++n>nsmps) {
           beg = p->beg2;
           end = p->end2;
           p->lphs = phs;
@@ -449,7 +486,7 @@ int loscil(CSOUND *csound, LOSC *p)
         }
         break;
       }
-      if (--nsmps) goto phsout;
+      if (LIKELY(++n<nsmps)) goto phsout;
       break;
     }
     p->lphs = phs;
@@ -458,9 +495,7 @@ int loscil(CSOUND *csound, LOSC *p)
  phsout:
     p->lphs = phs;
  put0:
-    do {
-      *ar1++ = FL(0.0);
-    } while (--nsmps);
+    memset(&ar1[n], '\0', sizeof(MYFLT)*(nsmps-n));
     return OK;
 
  phsck2:
@@ -468,57 +503,57 @@ int loscil(CSOUND *csound, LOSC *p)
       goto put0s;                               /* for STEREO:  */
     switch (p->curmod) {
     case 0:
-      do {                                      /* NO LOOPING  */
-        loscil_linear_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs += inc) >= end)
+      for (; n<nsmps; n++) {                    /* NO LOOPING  */
+        loscil_linear_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
+        if (UNLIKELY((phs += inc) >= end))
           goto nxtseg2;
-      } while (--nsmps);
+      }
       break;
     case 1:
-      do {                                      /* NORMAL LOOPING */
-        loscil_linear_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs += inc) >= end) {
+      for (; n<nsmps; n++) {                    /* NORMAL LOOPING */
+        loscil_linear_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
+        if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg2;
           phs -= end - beg;
         }
-      } while (--nsmps);
+      }
       break;
     case 2:
     case2s:
-      do {                                      /* BIDIR FORW, EVEN */
-        loscil_linear_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs += inc) >= end) {
+      for (; n<nsmps; n++) {                    /* BIDIR FORW, EVEN */
+        loscil_linear_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
+        if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg2;
           phs -= (phs - end) * 2;
           p->curmod = 3;
-          if (--nsmps) goto case3s;
+          if (++n<nsmps) goto case3s;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
     case 3:
     case3s:
-      do {                                      /* BIDIR BACK, EVEN */
-        loscil_linear_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs -= inc) < beg) {
+      for (; n<nsmps; n++) {                    /* BIDIR BACK, EVEN */
+       loscil_linear_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
+        if (UNLIKELY((phs -= inc) < beg)) {
           phs += (beg - phs) * 2;
           p->curmod = 2;
-          if (--nsmps) goto case2s;
+          if (++n<nsmps) goto case2s;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
 
     nxtseg2:
@@ -526,7 +561,7 @@ int loscil(CSOUND *csound, LOSC *p)
         p->seg1 = 0;
         if ((p->curmod = p->mod2) != 0)
           p->looping = 1;
-        if (--nsmps) {
+        if (++n<nsmps) {
           beg = p->beg2;
           end = p->end2;
           p->lphs = phs;
@@ -534,7 +569,7 @@ int loscil(CSOUND *csound, LOSC *p)
         }
         break;
       }
-      if (--nsmps) goto phsout2;
+      if (LIKELY(++n<nsmps)) goto phsout2;
       break;
     }
     p->lphs = phs;
@@ -543,11 +578,8 @@ int loscil(CSOUND *csound, LOSC *p)
  phsout2:
     p->lphs = phs;
  put0s:
-    do {
-      *ar1++ = FL(0.0);
-      *ar2++ = FL(0.0);
-    } while (--nsmps);
-
+    memset(&ar1[n], '\0', sizeof(MYFLT)*(nsmps-n));
+    memset(&ar2[n], '\0', sizeof(MYFLT)*(nsmps-n));
     return OK;
 }
 
@@ -555,15 +587,20 @@ int loscil3(CSOUND *csound, LOSC *p)
 {
     FUNC    *ftp;
     MYFLT   *ar1, *ar2, *ftbl, *xamp;
-    long    phs, inc, beg, end;
-    int     nsmps = csound->ksmps, aamp;
+    int32    phs, inc, beg, end;
+    uint32_t n = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nsmps = CS_KSMPS;
+    int     aamp;
+    MYFLT   xx;
 
     ftp = p->ftp;
     ftbl = ftp->ftable;
-    if ((inc = (long)(*p->kcps * p->cpscvt)) < 0)
+    if ((inc = (int32)(*p->kcps * p->cpscvt)) < 0)
       inc = -inc;
     xamp = p->xamp;
-    aamp = (p->XINCODE) ? 1 : 0;
+    xx = *xamp;
+    aamp = IS_ASIG_ARG(p->xamp) ? 1 : 0;
     if (p->seg1) {                      /* if still segment 1  */
       beg = p->beg1;
       end = p->end1;
@@ -576,62 +613,69 @@ int loscil3(CSOUND *csound, LOSC *p)
     }
     phs = p->lphs;
     ar1 = p->ar1;
+    if (UNLIKELY(n)) memset(ar1, '\0', n*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar1[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->stereo) {
       ar2 = p->ar2;
+      if (UNLIKELY(n)) memset(ar1, '\0', n*sizeof(MYFLT));
+      if (UNLIKELY(early)) memset(&ar2[nsmps], '\0', early*sizeof(MYFLT));
       goto phsck2;
     }
  phschk:
-    if (phs >= end && p->curmod != 3)
+    if (UNLIKELY(phs >= end && p->curmod != 3))
       goto put0;
     switch (p->curmod) {
     case 0:
-      do {                                      /* NO LOOPING  */
-        loscil_cubic_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs += inc) >= end)
+      for (; n<nsmps; n++) {                    /* NO LOOPING  */
+        loscil_cubic_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        if (UNLIKELY((phs += inc) >= end))
           goto nxtseg;
-      } while (--nsmps);
+      }
       break;
     case 1:
-      do {                                      /* NORMAL LOOPING */
-        loscil_cubic_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs += inc) >= end) {
+      for (; n<nsmps; n++) {                    /* NORMAL LOOPING */
+        loscil_cubic_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg;
           phs -= end - beg;
         }
-      } while (--nsmps);
+      }
       break;
     case 2:
     case2:
-      do {                                      /* BIDIR FORW, EVEN */
-        loscil_cubic_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs += inc) >= end) {
+      for (; n<nsmps; n++) {                    /* BIDIR FORW, EVEN */
+        loscil_cubic_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg;
           phs -= (phs - end) * 2;
           p->curmod = 3;
-          if (--nsmps) goto case3;
+          if (++n<nsmps) goto case3;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
     case 3:
     case3:
-      do {                                      /* BIDIR BACK, EVEN */
-        loscil_cubic_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs -= inc) < beg) {
+      for (; n<nsmps; n++) {                    /* BIDIR BACK, EVEN */
+        loscil_cubic_interp_mono(&ar1[n], ftbl, phs, ftp->flen);;
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        if (UNLIKELY((phs -= inc) < beg)) {
           phs += (beg - phs) * 2;
           p->curmod = 2;
-          if (--nsmps) goto case2;
+          if (++n<nsmps) goto case2;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
 
     nxtseg:
@@ -647,7 +691,7 @@ int loscil3(CSOUND *csound, LOSC *p)
         }
         break;
       }
-      if (--nsmps) goto phsout;
+      if (LIKELY(++n<nsmps)) goto phsout;
       break;
     }
     p->lphs = phs;
@@ -656,67 +700,68 @@ int loscil3(CSOUND *csound, LOSC *p)
  phsout:
     p->lphs = phs;
  put0:
-    do {
-      *ar1++ = FL(0.0);
-    } while (--nsmps);
+    memset(&ar1[n], 0, sizeof(MYFLT)*(nsmps-n));
+    /* do { */
+    /*   *ar1++ = FL(0.0); */
+    /* } while (--nsmps); */
     return OK;
 
  phsck2:
-    if (phs >= end && p->curmod != 3)
+    if (UNLIKELY(phs >= end && p->curmod != 3))
       goto put0s;                               /* for STEREO:  */
     switch (p->curmod) {
     case 0:
-      do {                                      /* NO LOOPING  */
-        loscil_cubic_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs += inc) >= end)
+      for (; n<nsmps; n++) {                    /* NO LOOPING  */
+        loscil_cubic_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
+        if (UNLIKELY((phs += inc) >= end))
           goto nxtseg2;
-      } while (--nsmps);
+      }
       break;
     case 1:
-      do {                                      /* NORMAL LOOPING */
-        loscil_cubic_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs += inc) >= end) {
+      for (; n<nsmps; n++) {                    /* NORMAL LOOPING */
+        loscil_cubic_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
+        if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg2;
           phs -= end - beg;
         }
-      } while (--nsmps);
+      }
       break;
     case 2:
     case2s:
-      do {                                      /* BIDIR FORW, EVEN */
-        loscil_cubic_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs += inc) >= end) {
+      for (; n<nsmps; n++) {                    /* BIDIR FORW, EVEN */
+        loscil_cubic_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
+        if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg2;
           phs -= (phs - end) * 2;
           p->curmod = 3;
-          if (--nsmps) goto case3s;
+          if (++n<nsmps) goto case3s;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
     case 3:
     case3s:
-      do {                                      /* BIDIR BACK, EVEN */
-        loscil_cubic_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
-        if ((phs -= inc) < beg) {
+      for (; n<nsmps; n++) {                    /* BIDIR BACK, EVEN */
+        loscil_cubic_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
+        if (UNLIKELY((phs -= inc) < beg)) {
           phs += (beg - phs) * 2;
           p->curmod = 2;
-          if (--nsmps) goto case2s;
+          if (++n<nsmps) goto case2s;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
 
     nxtseg2:
@@ -724,7 +769,7 @@ int loscil3(CSOUND *csound, LOSC *p)
         p->seg1 = 0;
         if ((p->curmod = p->mod2) != 0)
           p->looping = 1;
-        if (--nsmps) {
+        if (++n<nsmps) {
           beg = p->beg2;
           end = p->end2;
           p->lphs = phs;
@@ -732,7 +777,7 @@ int loscil3(CSOUND *csound, LOSC *p)
         }
         break;
       }
-      if (--nsmps) goto phsout2;
+      if (LIKELY(++n<nsmps)) goto phsout2;
       break;
     }
     p->lphs = phs;
@@ -741,46 +786,50 @@ int loscil3(CSOUND *csound, LOSC *p)
  phsout2:
     p->lphs = phs;
  put0s:
-    do {
-      *ar1++ = FL(0.0);
-      *ar2++ = FL(0.0);
-    } while (--nsmps);
-
+    memset(&ar1[n], '\0', sizeof(MYFLT)*(nsmps-n));
+    memset(&ar2[n], '\0', sizeof(MYFLT)*(nsmps-n));
     return OK;
 }
 
 #define ISINSIZ 32768L
 #define ADMASK  32767L
 
-int adset(CSOUND *csound, ADSYN *p)
+static int adset_(CSOUND *csound, ADSYN *p, int stringname)
 {
-    long    n;
+    int32    n;
     char    filnam[MAXNAME];
     MEMFIL  *mfp;
-    short   *adp, *endata, val;
+    int16   *adp, *endata, val;
     PTLPTR  *ptlap, *ptlfp, *ptlim;
     int     size;
 
     if (csound->isintab == NULL) {  /* if no sin table yet, make one */
-      short *ip;
-      csound->isintab = ip = (short*) mmalloc(csound, ISINSIZ * sizeof(short));
+      int16 *ip;
+      csound->isintab = ip =
+        (int16*) csound->Malloc(csound, ISINSIZ * sizeof(int16));
       for (n = 0; n < ISINSIZ; n++)
-        *ip++ = (short) (sin(TWOPI * n / ISINSIZ) * 32767.0);
+        *ip++ = (int16) (sin(TWOPI * n / ISINSIZ) * 32767.0);
     }
-    csound->strarg2name(csound, filnam, p->ifilcod, "adsyn.", p->XSTRCODE);
+    if (stringname) strncpy(filnam, ((STRINGDAT*)p->ifilcod)->data, MAXNAME-1);
+    else if (csound->ISSTRCOD(*p->ifilcod))
+      strncpy(filnam, get_arg_string(csound, *p->ifilcod), MAXNAME-1);
+    else csound->strarg2name(csound, filnam, p->ifilcod, "adsyn.", 0);
+
+
     if ((mfp = p->mfp) == NULL || strcmp(mfp->filename,filnam) != 0) {
       /* readfile if reqd */
-      if ((mfp = ldmemfile2(csound, filnam, CSFTYPE_HETRO)) == NULL) {
+      if (UNLIKELY((mfp = ldmemfile2withCB(csound, filnam,
+                                           CSFTYPE_HETRO, NULL)) == NULL)) {
         csound->InitError(csound, Str("ADSYN cannot load %s"), filnam);
         return NOTOK;
       }
       p->mfp = mfp;                         /*   & record         */
     }
 
-    adp = (short *) mfp->beginp;            /* align on file data */
-    endata = (short *) mfp->endp;
+    adp = (int16 *) mfp->beginp;            /* align on file data */
+    endata = (int16 *) mfp->endp;
     size = 1+(*adp == -1 ? MAXPTLS : *adp++); /* Old no header -> MAXPIL */
-    if (p->aux.auxp==NULL || p->aux.size < (long)sizeof(PTLPTR)*size)
+    if (p->aux.auxp==NULL || p->aux.size < (uint32_t)sizeof(PTLPTR)*size)
       csound->AuxAlloc(csound, sizeof(PTLPTR)*size, &p->aux);
 
     ptlap = ptlfp = (PTLPTR*)p->aux.auxp;   /* find base ptl blk */
@@ -790,12 +839,12 @@ int adset(CSOUND *csound, ADSYN *p)
         switch (val) {
         case -1:
           ptlap->nxtp = ptlap + 1;       /* chain the ptl blks */
-          if ((ptlap = ptlap->nxtp) >= ptlim) goto adsful;
+          if (UNLIKELY((ptlap = ptlap->nxtp) >= ptlim)) goto adsful;
           ptlap->ap = (DUPLE *) adp;     /*  record start amp  */
           ptlap->amp = ptlap->ap->val;
           break;
         case -2:
-          if ((ptlfp += 1) >= ptlim) goto adsful;
+          if (UNLIKELY((ptlfp += 1) >= ptlim)) goto adsful;
           ptlfp->fp = (DUPLE *) adp;     /*  record start frq  */
           ptlfp->frq = ptlfp->fp->val;
           ptlfp->phs = 0;                /*  and clr the phase */
@@ -806,7 +855,7 @@ int adset(CSOUND *csound, ADSYN *p)
         }
       }
     } while (adp < endata);
-    if (ptlap != ptlfp) {
+    if (UNLIKELY(ptlap != ptlfp)) {
       return csound->InitError(csound, Str("%d amp tracks, %d freq tracks"),
                                (int) (ptlap - (PTLPTR*)p->aux.auxp) - 1,
                                (int) (ptlfp - (PTLPTR*)p->aux.auxp) - 1);
@@ -820,33 +869,41 @@ int adset(CSOUND *csound, ADSYN *p)
     return csound->InitError(csound, Str("partial count exceeds MAXPTLS"));
 }
 
+int adset(CSOUND *csound, ADSYN *p){
+  return adset_(csound,p,0);
+}
+
+int adset_S(CSOUND *csound, ADSYN *p){
+  return adset_(csound,p,1);
+}
+
 #define ADSYN_MAXLONG FL(2147483647.0)
 
 int adsyn(CSOUND *csound, ADSYN *p)
 {
     PTLPTR  *curp, *prvp;
     DUPLE   *ap, *fp;
-    short   curtim, diff, ktogo;
-    long    phs, sinc, *sp, amp;
-    int     n, nsmps;
-    MYFLT   *ar;
+    int16   curtim, diff, ktogo;
+    int32   phs, sinc, amp;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    MYFLT   *ar = p->rslt;
     MYFLT   ampscale, frqscale;
-    long    timkincr, nxtim;
+    int32   timkincr, nxtim;
 
-    if (csound->isintab == NULL) {      /* RWD fix */
-      return csound->PerfError(csound, Str("adsyn: not initialised"));
+    if (UNLIKELY(csound->isintab == NULL)) {      /* RWD fix */
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("adsyn: not initialised"));
     }
     /* IV - Jul 11 2002 */
     ampscale = *p->kamod * csound->e0dbfs;      /* since 15-bit sine table */
     frqscale = *p->kfmod * ISINSIZ * csound->onedsr;
     /* 1024 * msecs of analysis */
-    timkincr = (long)(*p->ksmod*FL(1024000.0)*csound->onedkr);
-    sp = (long *) p->rslt;                      /* use out array for sums */
-    nsmps = csound->ksmps;
-    do {
-      *sp++ = 0L;                               /* cleared first to zero */
-    } while (--nsmps);
-    curtim = (short)(p->mksecs >> 10);          /* cvt mksecs to msecs */
+    memset(p->rslt,0,sizeof(MYFLT)*nsmps);
+    if (UNLIKELY(early)) nsmps -= early;
+    timkincr = (int32)(*p->ksmod*FL(1024000.0)*CS_ONEDKR);
+    curtim = (int16)(p->mksecs >> 10);          /* cvt mksecs to msecs */
     curp = (PTLPTR*)p->aux.auxp;                /* now for each partial:    */
     while ((prvp = curp) && (curp = curp->nxtp) != NULL ) {
       ap = curp->ap;
@@ -856,15 +913,14 @@ int adsyn(CSOUND *csound, ADSYN *p)
       while (curtim >= (fp+1)->tim)
         curp->fp = fp += 1;
       if ((amp = curp->amp)) {            /* for non-zero amp   */
-        sinc = (long)(curp->frq * frqscale);
+        sinc = (int32)(curp->frq * frqscale);
         phs = curp->phs;
-        sp = (long *) p->rslt;
-        nsmps = csound->ksmps;            /*   addin a sinusoid */
-        do {
-          *sp++ += csound->isintab[phs] * amp;
+        /*   addin a sinusoid */
+        for (n=offset; n<nsmps; n++) {
+          ar[n] += (ampscale*(MYFLT)csound->isintab[phs]*(MYFLT)amp)/ADSYN_MAXLONG;
           phs += sinc;
           phs &= ADMASK;
-        } while (--nsmps);
+        }
         curp->phs = phs;
       }
       if ((nxtim = (ap+1)->tim) == 32767) {   /* if last amp this partial */
@@ -872,28 +928,19 @@ int adsyn(CSOUND *csound, ADSYN *p)
         curp = prvp;
       }
       else {                                 /* else interp towds nxt amp */
-        if ((diff = (short)((ap+1)->val - amp))) {
-          ktogo = (short)(((nxtim<<10) - p->mksecs + timkincr - 1) / timkincr);
-          curp->amp += diff / ktogo;
+        if ((diff = (int16)((ap+1)->val - amp))) {
+          ktogo = (int16)(((nxtim<<10) - p->mksecs + timkincr - 1) / timkincr);
+          if (ktogo == 0) curp->amp += diff;
+          else            curp->amp += diff / ktogo;
         }
         if ((nxtim = (fp+1)->tim) != 32767            /*      & nxt frq */
             && (diff = (fp+1)->val - curp->frq)) {
-          ktogo = (short)(((nxtim<<10) - p->mksecs + timkincr - 1) / timkincr);
-          curp->frq += diff / ktogo;
+          ktogo = (int16)(((nxtim<<10) - p->mksecs + timkincr - 1) / timkincr);
+          if (ktogo == 0) curp->frq += diff;
+          else            curp->frq += diff / ktogo;
         }
       }
     }
     p->mksecs += timkincr;                  /* advance the time */
-    ar = p->rslt;
-    sp = (long *) ar;           /* Seems to make assumptions about long/MYFLT */
-    nsmps = csound->ksmps;
-      /* a quick-hack fix: should change adsyn to use floats table and
-         buffers and should replace hetro format anyway.... */
-    for (n=0; n<nsmps; n++)
-      ar[n] = (MYFLT) ((sp[n] * ampscale) / ADSYN_MAXLONG);
-/*     do { */
-/*       *ar++ = (MYFLT) ((*sp++ * ampscale) / ADSYN_MAXLONG); */
-/*     } while (--nsmps); */
     return OK;
 }
-
