@@ -24,12 +24,45 @@
 #include <dlfcn.h>
 #include <dirent.h>
 
+#undef CS_KSMPS
+#define CS_KSMPS     (csound->GetKsmps(csound))
+
 #ifdef BETA
 #define DEBUG 1
 #endif
 #define DSSI4CS_MAX_NUM_EVENTS 128
 
-static const char   *version = "0.1alpha";
+#if !defined(HAVE_STRLCAT) && !defined(strlcat)
+size_t
+strlcat(char *dst, const char *src, size_t siz)
+{
+    char *d = dst;
+    const char *s = src;
+    size_t n = siz;
+    size_t dlen;
+
+    /* Find the end of dst and adjust bytes left but don't go past end */
+    while (n-- != 0 && *d != '\0')
+      d++;
+    dlen = d - dst;
+    n = siz - dlen;
+
+    if (n == 0)
+      return (dlen + strlen(s));
+    while (*s != '\0') {
+      if (n != 1) {
+        *d++ = *s;
+        n--;
+      }
+      s++;
+    }
+    *d = '\0';
+
+    return (dlen + (s - src));  /* count does not include NUL */
+}
+#endif
+
+//static const char   *version = "0.1alpha";
 
 /* TODO accomodate plugins which return control outputs */
 
@@ -38,9 +71,10 @@ static const char   *version = "0.1alpha";
 *****************************************************************************/
 void info(CSOUND * csound, DSSI4CS_PLUGIN * DSSIPlugin_)
 {
-    int     Ksmps = csound->ksmps;
+    int     Ksmps = csound->GetKsmps(csound);
     unsigned long PortCount;
     LADSPA_Descriptor *Descriptor;
+    uint32 i;
 
     if (DSSIPlugin_->Type == LADSPA)
       Descriptor = (LADSPA_Descriptor *) DSSIPlugin_->Descriptor;
@@ -48,7 +82,6 @@ void info(CSOUND * csound, DSSI4CS_PLUGIN * DSSIPlugin_)
       Descriptor =
           (LADSPA_Descriptor *) DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin;
     PortCount = Descriptor->PortCount;
-    unsigned long i;
 
     csound->Message(csound, "============Plugin %i"
                             "========================================\n",
@@ -62,7 +95,7 @@ void info(CSOUND * csound, DSSI4CS_PLUGIN * DSSIPlugin_)
     csound->Message(csound, "Copyright: %s\n", Descriptor->Copyright);
     csound->Message(csound, "Number of Ports: %lu\n", PortCount);
     for (i = 0; i < PortCount; i++) {
-      csound->Message(csound, "  Port #%lu: %s %s: %s - Range: ", i,
+      csound->Message(csound, "  Port #%u: %s %s: %s - Range: ", i,
                       (LADSPA_IS_PORT_CONTROL(Descriptor->PortDescriptors[i]) ?
                        "Control" : "Audio"),
                       (LADSPA_IS_PORT_INPUT(Descriptor->PortDescriptors[i]) ?
@@ -118,7 +151,7 @@ DSSI4CS_PLUGIN *LocatePlugin(int PluginNumber, CSOUND * csound)
     if (!DSSIPlugin)
       return NULL;
     DSSIPlugin_ = DSSIPlugin;
-    if (PluginNumber > *DSSIPlugin->PluginCount) {
+    if (UNLIKELY(PluginNumber > *DSSIPlugin->PluginCount)) {
 #ifdef DEBUG
       csound->Message(csound,
                       "DSSI4CS: PluginNumber > *DSSIPlugin->PluginCount.\n");
@@ -132,7 +165,7 @@ DSSI4CS_PLUGIN *LocatePlugin(int PluginNumber, CSOUND * csound)
                       DSSIPlugin_->PluginNumber);
 #endif
 
-      if (!DSSIPlugin_->NextPlugin) {
+      if (UNLIKELY(!DSSIPlugin_->NextPlugin)) {
         return NULL;
       }
       DSSIPlugin_ = DSSIPlugin_->NextPlugin;
@@ -151,20 +184,22 @@ static int dssideinit(CSOUND * csound, DSSI4CS_PLUGIN * DSSIPlugin)
 
     for (i = 0; DSSIPlugin != NULL; i++) {
       DSSI4CS_PLUGIN *nxt = (DSSI4CS_PLUGIN *) DSSIPlugin->NextPlugin;
-      if (DSSIPlugin->Type == LADSPA) {
-        if (DSSIPlugin->Descriptor->deactivate != NULL)
-          DSSIPlugin->Descriptor->deactivate(DSSIPlugin->Handle);
-        if (DSSIPlugin->Descriptor->cleanup != NULL)
-          DSSIPlugin->Descriptor->cleanup(DSSIPlugin->Handle);
-      }
-      else {
-        if (DSSIPlugin->DSSIDescriptor->LADSPA_Plugin->deactivate != NULL)
-          DSSIPlugin->DSSIDescriptor->LADSPA_Plugin->deactivate(
-              DSSIPlugin->Handle);
-        if (DSSIPlugin->DSSIDescriptor->LADSPA_Plugin->cleanup != NULL)
-          DSSIPlugin->DSSIDescriptor->LADSPA_Plugin->cleanup(
-              DSSIPlugin->Handle);
+      if (DSSIPlugin->Descriptor) {
+        if (DSSIPlugin->Type == LADSPA) {
+          if (DSSIPlugin->Descriptor->deactivate != NULL)
+            DSSIPlugin->Descriptor->deactivate(DSSIPlugin->Handle);
+          if (DSSIPlugin->Descriptor->cleanup != NULL)
+            DSSIPlugin->Descriptor->cleanup(DSSIPlugin->Handle);
+        }
+        else {
+          if (DSSIPlugin->DSSIDescriptor->LADSPA_Plugin->deactivate != NULL)
+            DSSIPlugin->DSSIDescriptor->LADSPA_Plugin->
+              deactivate(DSSIPlugin->Handle);
+          if (DSSIPlugin->DSSIDescriptor->LADSPA_Plugin->cleanup != NULL)
+            DSSIPlugin->DSSIDescriptor->LADSPA_Plugin->cleanup(DSSIPlugin->Handle);
+        }
       }
+      else csound->Message(csound, "missing descriptor\n");
       if (i != 0)
         csound->Free(csound, DSSIPlugin);
       DSSIPlugin = nxt;
@@ -181,13 +216,13 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
 {
     /* TODO check if plugin has already been loaded and use same function */
     csound = p->h.insdshead->csound;
-    int     SampleRate = (int) MYFLT2LRND(csound->esr);
-    int     Ksmps = csound->ksmps;
-    int     i;
+    int     SampleRate = (int) MYFLT2LRND(csound->GetSr(csound));
+    int     Ksmps = csound->GetKsmps(csound);
+    unsigned long     i;
     LADSPA_Descriptor_Function pfDescriptorFunction;
     DSSI_Descriptor_Function pfDSSIDescriptorFunction;
     LADSPA_Descriptor *LDescriptor;
-    char    dssiFilename[0x100];
+    char    dssiFilename[MAXNAME];
     unsigned long PluginIndex;
     void   *PluginLibrary;
     unsigned long PortCount;
@@ -197,27 +232,23 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
     DSSI4CS_PLUGIN *DSSIPlugin_;
     DSSI4CS_PLUGIN *DSSIPlugin =
         (DSSI4CS_PLUGIN *) csound->QueryGlobalVariable(csound, "$DSSI4CS");
+    CS_TYPE* argType = csound->GetTypeForArg(p->iplugin);
 
-    if (!DSSIPlugin) {
-      csound->Message(csound, "=============================="
-                              "===============================\n");
-      csound->Message(csound, "dssi4cs version %s by Andres Cabrera\n",
-                      version);
-      csound->Message(csound,
-                      "Using code by Richard Furse from the LADSPA SDK.\n");
-      csound->Message(csound, "=============================="
-                              "===============================\n");
-    }
-    csound->strarg2name(csound, dssiFilename, p->iplugin, "dssiinit.",
-                                (int) csound->GetInputArgSMask(p));
+    if (strcmp("S", argType->varTypeName) == 0)
+      strncpy(dssiFilename,((STRINGDAT *)p->iplugin)->data, MAXNAME-1);
+    else
+      csound->strarg2name(csound, dssiFilename, csound->ISSTRCOD(*p->iplugin) ?
+                          csound->GetString(csound, *p->iplugin) :
+                          (char *) p->iplugin, "dssiinit.",
+                          (int) csound->ISSTRCOD(*p->iplugin));
     PluginIndex = (unsigned long) *p->iindex;
     PluginLibrary = dlopenLADSPA(csound, dssiFilename, RTLD_NOW);
-    if (!PluginLibrary)
+    if (UNLIKELY(!PluginLibrary))
       return csound->InitError(csound, "DSSI4CS: Failed to load %s.",
                                        dssiFilename);
     if (!DSSIPlugin) {
-      if (csound->CreateGlobalVariable(csound, "$DSSI4CS",
-                                               sizeof(DSSI4CS_PLUGIN)) != 0)
+      if (UNLIKELY(csound->CreateGlobalVariable(csound, "$DSSI4CS",
+                                                sizeof(DSSI4CS_PLUGIN))) != 0)
         csound->Die(csound, "Error creating global variable '$DSSI4CS'");
       DSSIPlugin = (DSSI4CS_PLUGIN *) csound->QueryGlobalVariable(csound,
                                                                   "$DSSI4CS");
@@ -227,16 +258,16 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
       DSSIPlugin->PluginCount = (int *) csound->Malloc(csound, sizeof(int));
       *DSSIPlugin->PluginCount = 1;
       DSSIPlugin_ = DSSIPlugin;
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: Loading first instance.\n");
-#endif
+      if (p->iverbose != 0) {
+        csound->Message(csound, "DSSI4CS: Loading first instance.\n");
+      }
     }
     else {
       DSSIPlugin_ = LocatePlugin(*DSSIPlugin->PluginCount - 1, csound);
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: Located plugin: %i.\n",
-                      DSSIPlugin_->PluginNumber);
-#endif
+      if (p->iverbose != 0) {
+        csound->Message(csound, "DSSI4CS: Located plugin: %i.\n",
+                        DSSIPlugin_->PluginNumber);
+      }
       DSSIPlugin_->NextPlugin =
           (DSSI4CS_PLUGIN *) csound->Malloc(csound, sizeof(DSSI4CS_PLUGIN));
       DSSIPlugin_ = DSSIPlugin_->NextPlugin;
@@ -245,11 +276,11 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
       *DSSIPlugin_->PluginCount = (*DSSIPlugin_->PluginCount) + 1;
     }
     *p->iDSSIHandle = DSSIPlugin_->PluginNumber;
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: About to load descriptor function "
-                            "for plugin %i of %i.\n",
-                    DSSIPlugin_->PluginNumber, *DSSIPlugin_->PluginCount);
-#endif
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: About to load descriptor function "
+                      "for plugin %i of %i.\n",
+                      DSSIPlugin_->PluginNumber, *DSSIPlugin_->PluginCount);
+    }
     pfDSSIDescriptorFunction =
         (DSSI_Descriptor_Function) dlsym(PluginLibrary, "dssi_descriptor");
     if (pfDSSIDescriptorFunction) {
@@ -260,14 +291,17 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
       LDescriptor =
           (LADSPA_Descriptor *) DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin;
       DSSIPlugin_->Type = DSSI;
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: DSSI Plugin detected.\n");
-#endif
+      if (p->iverbose != 0) {
+        csound->Message(csound, "DSSI4CS: DSSI Plugin detected.\n");
+      }
     }
     else {
       pfDescriptorFunction =
           (LADSPA_Descriptor_Function) dlsym(PluginLibrary,
                                              "ladspa_descriptor");
+      if (pfDescriptorFunction==NULL) {
+        return csound->InitError(csound, "No lapspa descriptor\n");
+      }
       DSSIPlugin_->Descriptor =
           (LADSPA_Descriptor *) csound->Calloc(csound,
                                                sizeof(LADSPA_Descriptor));
@@ -275,65 +309,66 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
           (LADSPA_Descriptor *) pfDescriptorFunction(PluginIndex);
       LDescriptor = (LADSPA_Descriptor *) DSSIPlugin_->Descriptor;
       DSSIPlugin_->Type = LADSPA;
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: LADSPA Plugin Detected\n");
-#endif
+      if (p->iverbose != 0) {
+        csound->Message(csound, "DSSI4CS: LADSPA Plugin Detected\n");
+      }
     }
-    if ((!DSSIPlugin_->Descriptor) && (!DSSIPlugin_->DSSIDescriptor)) {
+    if (UNLIKELY((!DSSIPlugin_->Descriptor) &&
+                 (!DSSIPlugin_->DSSIDescriptor))) {
       const char *pcError = dlerror();
 
       /* TODO: cleanup if error */
       /* csound->Free(csound, DSSIPlugin_->Descriptor); */
       if (pcError)
-        csound->InitError(csound, "DSSI4CS: Unable to find "
-                                  "ladspa_descriptor() function or\n"
-                                  "dssi_descriptor() function in plugin file "
-                                  "\"%s\": %s.\n"
-                                  "Are you sure this is a LADSPA or "
-                                  "DSSI plugin file ?",
-                                  dssiFilename, pcError);
+        return
+          csound->InitError(csound, "DSSI4CS: Unable to find "
+                            "ladspa_descriptor() function or\n"
+                            "dssi_descriptor() function in plugin file "
+                            "\"%s\": %s.\n"
+                            "Are you sure this is a LADSPA or "
+                            "DSSI plugin file ?",
+                            dssiFilename, pcError);
       else
-        csound->InitError(csound, "DSSI4CS: Unable to find "
-                                  "ladspa_descriptor() function or\n"
-                                  "dssi_descriptor() function in plugin file "
-                                  "\"%s\".\n"
-                                  "Are you sure this is a LADSPA or "
-                                  "DSSI plugin file ?",
-                                  dssiFilename);
-      return NOTOK;
+        return
+          csound->InitError(csound, "DSSI4CS: Unable to find "
+                            "ladspa_descriptor() function or\n"
+                            "dssi_descriptor() function in plugin file "
+                            "\"%s\".\n"
+                            "Are you sure this is a LADSPA or "
+                            "DSSI plugin file ?",
+                            dssiFilename);
     }
-    if (!LDescriptor) {
-      csound->InitError(csound, "DSSI4CS: No plugin index %lu in %s",
-                                PluginIndex, dssiFilename);
-      return NOTOK;
+    if (UNLIKELY(!LDescriptor)) {
+      return csound->InitError(csound, "DSSI4CS: No plugin index %lu in %s",
+                               PluginIndex, dssiFilename);
+    }
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: About to instantiate plugin.\n");
     }
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: About to instantiate plugin.\n");
-#endif
 
     if (DSSIPlugin_->Type == LADSPA) {
-      if (!
+      if (UNLIKELY(!
           (DSSIPlugin_->Handle =
            (LADSPA_Handle) DSSIPlugin_->Descriptor->instantiate(DSSIPlugin_->
                                                                 Descriptor,
-                                                                SampleRate)))
+                                                                SampleRate))))
         return csound->InitError(csound,
                                  "DSSI4CS: Could not instantiate plugin: %s",
                                  dssiFilename);
-      if (!DSSIPlugin_->Descriptor->run)
+      if (UNLIKELY(!DSSIPlugin_->Descriptor->run))
         return csound->InitError(csound, "DSSI4CS: No run() funtion in: %s",
                                          LDescriptor->Name);
       PortCount = DSSIPlugin_->Descriptor->PortCount;
     }
     else {
-      if (!
+      if (UNLIKELY(!
           (DSSIPlugin_->Handle =
            (LADSPA_Handle) DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->
-           instantiate(DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin, SampleRate)))
+           instantiate(DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin, SampleRate))))
         return csound->InitError(csound,
                                  "DSSI4CS: Could not instantiate plugin: %s",
                                  dssiFilename);
-      if (!DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->run)
+      if (UNLIKELY(!DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->run))
         return csound->InitError(csound, "DSSI4CS: No run() funtion in: %s",
                                          LDescriptor->Name);
       PortCount = DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->PortCount;
@@ -342,12 +377,12 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
                                              DSSI4CS_MAX_NUM_EVENTS
                                              * sizeof(snd_seq_event_t));
     }
-#ifdef DEBUG
-    if (DSSIPlugin_->Handle)
-      csound->Message(csound, "DSSI4CS: Plugin instantiated.\n");
-    else
-      csound->Message(csound, "DSSI4CS: Problem instantiating.\n");
-#endif
+    if (p->iverbose != 0) {
+      if (DSSIPlugin_->Handle)
+        csound->Message(csound, "DSSI4CS: Plugin instantiated.\n");
+      else
+        csound->Message(csound, "DSSI4CS: Problem instantiating.\n");
+    }
 
     for (i = 0; i < PortCount; i++) {
       PortDescriptor =
@@ -360,12 +395,12 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
       if (LADSPA_IS_PORT_AUDIO(PortDescriptor))
         ConnectedAudioPorts++;
     }
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: Found %lu control ports for: '%s'\n",
-                    ConnectedControlPorts, LDescriptor->Name);
-    csound->Message(csound, "DSSI4CS: Found %lu audio ports for: '%s'\n",
-                    ConnectedAudioPorts, LDescriptor->Name);
-#endif
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: Found %lu control ports for: '%s'\n",
+                      ConnectedControlPorts, LDescriptor->Name);
+      csound->Message(csound, "DSSI4CS: Found %lu audio ports for: '%s'\n",
+                      ConnectedAudioPorts, LDescriptor->Name);
+    }
 
     DSSIPlugin_->control =
         (LADSPA_Data **) csound->Calloc(csound, ConnectedControlPorts
@@ -373,9 +408,9 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
     DSSIPlugin_->audio =
         (LADSPA_Data **) csound->Calloc(csound, ConnectedAudioPorts
                                                 * sizeof(LADSPA_Data *));
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: Created port array.\n");
-#endif
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: Created port array.\n");
+    }
 
     ConnectedControlPorts = 0;
     ConnectedAudioPorts = 0;
@@ -384,27 +419,30 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
           (DSSIPlugin_->Type ==
            LADSPA ? DSSIPlugin_->Descriptor->PortDescriptors[i]
            : DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->PortDescriptors[i]);
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: Queried port descriptor.\n");
-#endif
+      if (p->iverbose != 0) {
+        csound->Message(csound, "DSSI4CS: Queried port descriptor.\n");
+      }
 
       if (LADSPA_IS_PORT_CONTROL(PortDescriptor)) {
         DSSIPlugin_->control[ConnectedControlPorts] =
             (LADSPA_Data *) csound->Calloc(csound, sizeof(LADSPA_Data));
-        if (DSSIPlugin_->Type == LADSPA)
+        if (DSSIPlugin_->Type == LADSPA) {
           DSSIPlugin_->Descriptor->connect_port(DSSIPlugin_->Handle,
                                                 i,
                                                 (LADSPA_Data *) DSSIPlugin_->
                                                 control[ConnectedControlPorts]);
-        else
+        }
+        else {
           DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->connect_port(
               DSSIPlugin_->Handle, i,
               (LADSPA_Data *) DSSIPlugin_->control[ConnectedControlPorts]);
-#ifdef DEBUG
-        csound->Message(csound,
-                        "DSSI4CS: Created control port %lu for Port %i.\n",
-                        ConnectedControlPorts, i);
-#endif
+        }
+        if (p->iverbose != 0) {
+          csound->Message(csound,
+                          "DSSI4CS: Created internal control port "
+                          "%lu for Port %lu.\n",
+                          ConnectedControlPorts, i);
+        }
 
         ConnectedControlPorts++;
       }
@@ -420,32 +458,33 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
           DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->connect_port(
               DSSIPlugin_->Handle, i,
               (LADSPA_Data *) DSSIPlugin_->audio[ConnectedAudioPorts]);
-#ifdef DEBUG
-        csound->Message(csound,
-                        "DSSI4CS: Created audio port %lu for Port %i.\n",
-                        ConnectedAudioPorts, i);
-#endif
+        if (p->iverbose != 0) {
+          csound->Message(csound,
+                          "DSSI4CS: Created internal audio port"
+                          " %lu for Port %lu.\n",
+                          ConnectedAudioPorts, i);
+        }
 
         ConnectedAudioPorts++;
       }
 
     }
     /* All ports must be connected before calling run() */
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: Created %lu control ports for: '%s'\n",
-                    ConnectedControlPorts, LDescriptor->Name);
-    csound->Message(csound, "DSSI4CS: Created %lu audio ports for: '%s'\n",
-                    ConnectedAudioPorts, LDescriptor->Name);
-#endif
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: Created %lu control ports for: '%s'\n",
+                      ConnectedControlPorts, LDescriptor->Name);
+      csound->Message(csound, "DSSI4CS: Created %lu audio ports for: '%s'\n",
+                      ConnectedAudioPorts, LDescriptor->Name);
+    }
 
     DSSIPlugin_->Active = 0;
     DSSIPlugin_->EventCount = 0;
-    if (*p->iverbose != 0)
-      info(csound, DSSIPlugin_);
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: Init Done.\n");
-#endif
-
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: Init Done.\n");
+      if (*p->iverbose != 0)
+        info(csound, DSSIPlugin_);
+    }
+    dlclose(PluginLibrary);
     return OK;
 }
 
@@ -507,8 +546,8 @@ int dssiactivate_init(CSOUND * csound, DSSIACTIVATE * p)
 
     p->DSSIPlugin_ = LocatePlugin(Number, csound);
     p->printflag = -999;
-    if ((!p->DSSIPlugin_) || (Number > *p->DSSIPlugin_->PluginCount) ||
-        (!p->DSSIPlugin_->Handle))
+    if (UNLIKELY((!p->DSSIPlugin_) || (Number > *p->DSSIPlugin_->PluginCount) ||
+                 (!p->DSSIPlugin_->Handle)))
       return csound->InitError(csound, "DSSI4CS: Invalid plugin: %i (MAX= %i).",
                                        Number, *p->DSSIPlugin_->PluginCount);
 #ifdef DEBUG
@@ -572,11 +611,11 @@ int dssiactivate(CSOUND * csound, DSSIACTIVATE * p)
       break;
     case -100:
       {
-        if (p->printflag != -100) {
-          return csound->PerfError(csound, "DSSI4CS: dssiactivate "
-                                           "not properly initialised.");
-          p->printflag = -100;
-        }
+        if (p->printflag != -100)
+          return csound->PerfError(csound, p->h.insdshead,
+                                   "DSSI4CS: dssiactivate "
+                                   "not properly initialised.");
+        p->printflag = -100;
       }
       break;
 
@@ -593,6 +632,20 @@ int dssiaudio_init(CSOUND * csound, DSSIAUDIO * p)
 {
     /* TODO not realtime safe, try to make it so. */
     int     Number = *p->iDSSIhandle;
+    int     icnt = csound->GetInputArgCnt(p) - 1;
+    int     ocnt = csound->GetOutputArgCnt(p);
+
+    if (UNLIKELY(icnt > DSSI4CS_MAX_IN_CHANNELS))
+      csound->Die(csound,
+                  Str("DSSI4CS: number of audio input channels "
+                      "is greater than %d"),
+                  DSSI4CS_MAX_IN_CHANNELS);
+
+    if (UNLIKELY(ocnt > DSSI4CS_MAX_OUT_CHANNELS))
+      csound->Die(csound,
+                  Str("DSSI4CS: number of audio output channels "
+                      "is greater than %d"),
+                  DSSI4CS_MAX_OUT_CHANNELS);
 
 #ifdef DEBUG
     csound->Message(csound,
@@ -614,10 +667,10 @@ int dssiaudio_init(CSOUND * csound, DSSIAUDIO * p)
       Descriptor =
           (LADSPA_Descriptor *) p->DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin;
 
-    long    PortIndex = 0;
-    int     ConnectedInputPorts = 0;
-    int     ConnectedOutputPorts = 0;
-    int     ConnectedPorts = 0;
+    unsigned long PortIndex = 0;
+    int           ConnectedInputPorts = 0;
+    int           ConnectedOutputPorts = 0;
+    int           ConnectedPorts = 0;
     LADSPA_PortDescriptor PortDescriptor = 0;
 
     for (PortIndex = 0; PortIndex < Descriptor->PortCount; PortIndex++) {
@@ -637,10 +690,6 @@ int dssiaudio_init(CSOUND * csound, DSSIAUDIO * p)
     ConnectedOutputPorts = 0;
     ConnectedPorts = 0;
     for (PortIndex = 0; PortIndex < Descriptor->PortCount; PortIndex++) {
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: Port Index: %lu\n", PortIndex);
-#endif
-
       PortDescriptor = Descriptor->PortDescriptors[PortIndex];
       if (LADSPA_IS_PORT_INPUT(PortDescriptor)
           && LADSPA_IS_PORT_AUDIO(PortDescriptor)) {
@@ -679,7 +728,7 @@ int dssiaudio_init(CSOUND * csound, DSSIAUDIO * p)
 
     p->NumInputPorts = ConnectedInputPorts;
     p->NumOutputPorts = ConnectedOutputPorts;
-    if ((p->NumInputPorts) < (csound->GetInputArgCnt(p) - 1)) {
+    if ((p->NumInputPorts) < icnt) {
       if (p->NumInputPorts == 0)
         csound->Message(csound, "DSSI4CS: Plugin '%s' has %i audio input ports "
                                 "audio input discarded.\n",
@@ -689,7 +738,7 @@ int dssiaudio_init(CSOUND * csound, DSSIAUDIO * p)
                                          "has %i audio input ports.",
                                          Descriptor->Name, p->NumOutputPorts);
     }
-    if (p->NumOutputPorts < csound->GetOutputArgCnt(p))
+    if (p->NumOutputPorts < ocnt)
       return csound->InitError(csound, "DSSI4CS: Plugin '%s' "
                                        "has %i audio output ports.",
                                        Descriptor->Name, p->NumOutputPorts);
@@ -705,22 +754,22 @@ int dssiaudio(CSOUND * csound, DSSIAUDIO * p)
     else
       Descriptor =
           (LADSPA_Descriptor *) p->DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin;
-    int     i, j;
-    int     icnt = csound->GetInputArgCnt(p) - 1;
-    int     ocnt = csound->GetOutputArgCnt(p);
-    unsigned long Ksmps = (unsigned long) csound->ksmps;
+    unsigned int i, j;
+    unsigned int icnt = csound->GetInputArgCnt(p) - 1;
+    unsigned int ocnt = csound->GetOutputArgCnt(p);
+    unsigned long Ksmps = (unsigned long) csound->GetKsmps(csound);
 
     if (p->DSSIPlugin_->Active == 1) {
       for (j = 0; j < icnt; j++) {
         for (i = 0; i < Ksmps; i++)
           p->DSSIPlugin_->audio[p->InputPorts[j]][i] =
-              p->ain[j][i] * csound->dbfs_to_float;
+            p->ain[j][i] * (1.0/csound->Get0dBFS(csound));
       }
       Descriptor->run(p->DSSIPlugin_->Handle, Ksmps);
       for (j = 0; j < ocnt; j++) {
         for (i = 0; i < Ksmps; i++)
           p->aout[j][i] =
-              p->DSSIPlugin_->audio[p->OutputPorts[j]][i] * csound->e0dbfs;
+            p->DSSIPlugin_->audio[p->OutputPorts[j]][i] * csound->Get0dBFS(csound);
       }
     }
     else {
@@ -738,17 +787,19 @@ int dssiaudio(CSOUND * csound, DSSIAUDIO * p)
 int dssictls_kk(CSOUND * csound, DSSICTLS * p)
 {
     LADSPA_Data Value = *p->val;
-
-    if (!p->DSSIPlugin_)
-      return csound->PerfError(csound, "DSSI4CS: Invalid plugin handle.");
-    if (*p->ktrig == 1)
+    if (!p->DSSIPlugin_) {
+      return csound->PerfError(csound, p->h.insdshead,
+                               "DSSI4CS: Invalid plugin handle.");
+    }
+    if (*p->ktrig == 1) {
       *p->DSSIPlugin_->control[p->PortNumber] = Value * (p->HintSampleRate);
+    }
     return OK;
 }
 
 int dssictls_ak(CSOUND * csound, DSSICTLS * p)
 {
-    csound->PerfError(csound,
+    csound->PerfError(csound, p->h.insdshead,
                       "DSSI4CS: Audio Rate control ports not implemented yet.");
     return NOTOK;
 }
@@ -759,21 +810,24 @@ int dssictls_init(CSOUND * csound, DSSICTLS * p)
             Crash if audio port selected */
     const LADSPA_Descriptor *Descriptor;
     int     Number = *p->iDSSIhandle;
-    int     Sr = (int) MYFLT2LRND(csound->esr);
+    int     Sr = (int) MYFLT2LRND(csound->GetSr(csound));
     unsigned long PortIndex = *p->iport;
-    int     i;
+    unsigned int  i;
     unsigned long ControlPort = 0;
     unsigned long AudioPort = 0;
     unsigned long Port = 0;
 
     p->DSSIPlugin_ = LocatePlugin(Number, csound);
-    if (!p->DSSIPlugin_)
+    if (!p->DSSIPlugin_) {
       return csound->InitError(csound, "DSSI4CS: Invalid plugin handle.");
-    if (p->DSSIPlugin_->Type == LADSPA)
+    }
+    if (p->DSSIPlugin_->Type == LADSPA) {
       Descriptor = (LADSPA_Descriptor *) p->DSSIPlugin_->Descriptor;
-    else
+    }
+    else {
       Descriptor =
           (LADSPA_Descriptor *) p->DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin;
+    }
     p->HintSampleRate =
         (LADSPA_IS_HINT_SAMPLE_RATE
          (Descriptor->PortRangeHints[PortIndex].HintDescriptor) ? Sr : 1);
@@ -794,37 +848,38 @@ int dssictls_init(CSOUND * csound, DSSICTLS * p)
       return csound->InitError(csound,
                                "DSSI4CS: Port %lu from '%s' is an output port.",
                                PortIndex, Descriptor->Name);
-    if (!LADSPA_IS_PORT_CONTROL(PortDescriptor))
-      for (i = 0; i < PortIndex; i++) {
-        PortDescriptor = Descriptor->PortDescriptors[i];
-        if (LADSPA_IS_PORT_CONTROL(PortDescriptor)) {
-          ControlPort++;
-          Port = ControlPort;
-          if (LADSPA_IS_PORT_AUDIO(PortDescriptor)) {
-            AudioPort++;
-            Port = AudioPort;
-          }
-        }
+    for (i = 0; i < PortIndex; i++) {
+      PortDescriptor = Descriptor->PortDescriptors[i];
+      if (LADSPA_IS_PORT_CONTROL(PortDescriptor)) {
+        ControlPort++;
+        Port = ControlPort;
       }
+      if (LADSPA_IS_PORT_AUDIO(PortDescriptor)) {
+        AudioPort++;
+        Port = AudioPort;
+      }
+    }
     p->PortNumber = Port;
 #ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: Connected Plugport %lu to output %lu.\n",
-                    PortIndex, ControlPort);
-    csound->Message(csound, "DSSI4CS: ArgMask: %lu.\n",
-                    csound->GetInputArgAMask(p));
+    csound->Message(csound, "DSSI4CS: Port %lu using internal port %lu.\n",
+                    PortIndex, p->PortNumber);
+    /*csound->Message(csound, "DSSI4CS: ArgMask: %lu.\n",*/
+    /*                csound->GetInputArgAMask(p));*/
 #endif
 
-    if ((int) csound->GetInputArgAMask(p) & 4)
-      p->h.opadr = (SUBR) dssictls_ak;  /* "iiak" */
-    else
-      p->h.opadr = (SUBR) dssictls_kk;  /* "iikk" */
+//    if ((int) csound->GetInputArgAMask(p) & 4) {
+//      p->h.opadr = (SUBR) dssictls_ak;  /* "iiak" */
+//    }
+//    else {
+//      p->h.opadr = (SUBR) dssictls_kk;  /* "iikk" */
+//    }
 
     return OK;
 }
 
 int dssictls_dummy(CSOUND * csound, DSSICTLS * p)
 {
-    csound->PerfError(csound,
+    csound->PerfError(csound, p->h.insdshead,
                       Str("DSSI4CS: Not initialised or wrong argument types."));
     return NOTOK;
 }
@@ -883,6 +938,7 @@ static void
     DIR    *psDirectory;
     LADSPA_Descriptor_Function fDescriptorFunction;
     long    lDirLength;
+    long    slen;
     long    iNeedSlash;
     struct dirent *psDirectoryEntry;
     void   *pvPluginHandle;
@@ -903,16 +959,17 @@ static void
       psDirectoryEntry = readdir(psDirectory);
       if (!psDirectoryEntry) {
         closedir(psDirectory);
+        //if (pvPluginHandle) closedir(pvPluginHandle);
         return;
       }
 
-      pcFilename = csound->Malloc(csound,
-                                  lDirLength + strlen(psDirectoryEntry->d_name)
-                                             + 1 + iNeedSlash);
-      strcpy(pcFilename, pcDirectory);
+      pcFilename =
+        csound->Malloc(csound,
+                       slen = (lDirLength + strlen(psDirectoryEntry->d_name) + 2));
+      strncpy(pcFilename, pcDirectory, slen); /*pcFilename[slen-1] = '\0';*/
       if (iNeedSlash)
-        strcat(pcFilename, "/");
-      strcat(pcFilename, psDirectoryEntry->d_name);
+        strlcat(pcFilename, "/",slen);
+      strlcat(pcFilename, psDirectoryEntry->d_name, slen);
 
       pvPluginHandle = dlopen(pcFilename, RTLD_LAZY);
       if (pvPluginHandle) {
@@ -926,13 +983,14 @@ static void
              it to the callback function. */
           fCallbackFunction(csound,
                             pcFilename, pvPluginHandle, fDescriptorFunction);
-          csound->Free(csound, pcFilename);
         }
         else {
           /* It was a library, but not a LADSPA one. Unload it. */
-          dlclose(pcFilename);
-          csound->Free(csound, pcFilename);
+          dlclose(pvPluginHandle);
+          pvPluginHandle = NULL;
+          //csound->Free(csound, pcFilename);
         }
+        csound->Free(csound, pcFilename);
       }
     }
 }
@@ -943,7 +1001,7 @@ LADSPAPluginSearch(CSOUND *csound,
 {
     char   *pcBuffer;
     const char *pcEnd;
-    const char *pcLADSPAPath;
+          char *pcLADSPAPath;
     const char *pcDSSIPath;
     const char *pcStart;
 
@@ -952,28 +1010,34 @@ LADSPAPluginSearch(CSOUND *csound,
     if (!pcLADSPAPath) {
       csound->Message(csound,
                       "DSSI4CS: LADSPA_PATH environment variable not set.\n");
+#ifdef LIB64
+      pcLADSPAPath = "/usr/lib64/ladspa/";
+#else
+      pcLADSPAPath = "/usr/lib/ladspa/";
+#endif
     }
     if (!pcDSSIPath) {
       csound->Message(csound,
                       "DSSI4CS: DSSI_PATH environment variable not set.\n");
+      pcStart = pcLADSPAPath;
     }
-    if ((!pcLADSPAPath) && (!pcLADSPAPath))
-      return;
-    if (pcDSSIPath) {
-      pcLADSPAPath = strcat((char *) pcLADSPAPath, ":");
-      pcLADSPAPath = strcat((char *) pcLADSPAPath, pcDSSIPath);
+    else {
+      int len = strlen(pcLADSPAPath)+strlen(pcDSSIPath)+2;
+      char *tmp = (char*)malloc(len);
+      snprintf(tmp, len, "%s:%s", pcLADSPAPath, pcDSSIPath);
+      pcLADSPAPath = pcStart = (const char*)tmp;
     }
-    pcStart = pcLADSPAPath;
+    // Search the list
     while (*pcStart != '\0') {
       pcEnd = pcStart;
       while (*pcEnd != ':' && *pcEnd != '\0')
         pcEnd++;
 
       pcBuffer = csound->Malloc(csound, 1 + (pcEnd - pcStart));
-      if (pcEnd > pcStart)
+      if (pcEnd > pcStart) {
         strncpy(pcBuffer, pcStart, pcEnd - pcStart);
-      pcBuffer[pcEnd - pcStart] = '\0';
-
+        pcBuffer[pcEnd - pcStart] = '\0';
+      }
       LADSPADirectoryPluginSearch(csound, pcBuffer, fCallbackFunction);
       csound->Free(csound, pcBuffer);
 
@@ -981,6 +1045,7 @@ LADSPAPluginSearch(CSOUND *csound,
       if (*pcStart == ':')
         pcStart++;
     }
+    if (pcDSSIPath) free(pcLADSPAPath);
 }
 
 void
@@ -1008,7 +1073,7 @@ int dssilist(CSOUND * csound, DSSILIST * p)
     /* Most of this function comes from the ladspa sdk by Richard Furse */
     char   *pcBuffer;
     const char *pcEnd;
-    const char *pcLADSPAPath;
+          char *pcLADSPAPath;
     const char *pcDSSIPath;
     const char *pcStart;
 
@@ -1022,11 +1087,20 @@ int dssilist(CSOUND * csound, DSSILIST * p)
       csound->Message(csound,
                       "DSSI4CS: DSSI_PATH environment variable not set.\n");
     }
-    if ((!pcLADSPAPath) && (!pcLADSPAPath))
+    if ((!pcLADSPAPath) && (!pcDSSIPath)) /* Fixed - JPff */
       return NOTOK;
-    if (pcDSSIPath) {
-      pcLADSPAPath = strcat((char *) pcLADSPAPath, ":");
-      pcLADSPAPath = strcat((char *) pcLADSPAPath, pcDSSIPath);
+    if (pcDSSIPath) {  /* **** THIS CODE WAS WRONG -- NO SPACEALLOCATED **** */
+      if (pcLADSPAPath) {
+        char *nn =
+          (char*)malloc(strlen((char *) pcLADSPAPath)+strlen(pcDSSIPath)+2);
+        strcpy(nn, pcLADSPAPath);
+        strcat(nn, ":");
+        strcat(nn, pcDSSIPath);
+        //free(pcLADSPAPath);
+        pcLADSPAPath = nn;
+      }
+      else pcLADSPAPath = strdup(pcDSSIPath);
+
     }
     pcStart = pcLADSPAPath;
     while (*pcStart != '\0') {
@@ -1043,34 +1117,35 @@ int dssilist(CSOUND * csound, DSSILIST * p)
       if (*pcStart == ':')
         pcStart++;
     }
+    if (pcDSSIPath) free(pcLADSPAPath);
     return OK;
 }
 
-static OENTRY localops[] = {
-    {"dssiinit", sizeof(DSSIINIT), 1, "i", "Tip", (SUBR) dssiinit, NULL, NULL }
+static OENTRY dssi_localops[] = {
+  {"dssiinit", sizeof(DSSIINIT), 0, 1, "i", "Tip", (SUBR) dssiinit, NULL, NULL }
     ,
-    {"dssiactivate", sizeof(DSSIACTIVATE), 3, "", "ik",
+    {"dssiactivate", sizeof(DSSIACTIVATE), 0, 3, "", "ik",
      (SUBR) dssiactivate_init, (SUBR) dssiactivate, NULL }
     ,
-    {"dssiaudio", sizeof(DSSIAUDIO), 5, "mmmm", "iy", (SUBR) dssiaudio_init,
-     NULL, (SUBR) dssiaudio }
+    {"dssiaudio", sizeof(DSSIAUDIO), 0, 5, "mmmmmmmmm", "iMMMMMMMMM",
+                                            (SUBR) dssiaudio_init,
+                                            NULL, (SUBR) dssiaudio }
     ,
-    {"dssictls", sizeof(DSSICTLS), 7, "", "iixk", (SUBR) dssictls_init,
-     (SUBR) dssictls_dummy, (SUBR) dssictls_dummy }
+    {"dssictls", sizeof(DSSICTLS), 0, 3, "", "iikk", (SUBR) dssictls_init,
+     (SUBR) dssictls_kk, NULL }
     ,
-    {"dssilist", sizeof(DSSILIST), 1, "", "", (SUBR) dssilist, NULL, NULL }
+    {"dssilist", sizeof(DSSILIST), 0, 1, "", "", (SUBR) dssilist, NULL, NULL }
 #if 0
     ,
-    {"dssisynth", sizeof(DSSISYNTH), 5, "aa",  "i", (SUBR) dssisynth_init,
+    {"dssisynth", sizeof(DSSISYNTH), 0, 5, "aa",  "i", (SUBR) dssisynth_init,
      NULL, (SUBR) dssisynth}
     ,
-    {"dssinote", sizeof(DSSINOTE), 3, "",  "kikkk", (SUBR) dssinote_init,
+    {"dssinote", sizeof(DSSINOTE), 0, 3, "",  "kikkk", (SUBR) dssinote_init,
      NULL, (SUBR) dssinote}
     ,
-    {"dssievent", sizeof(DSSINOTEON), 3, "",  "kikk", (SUBR) dssievent_init,
+    {"dssievent", sizeof(DSSINOTEON), 0, 3, "",  "kikk", (SUBR) dssievent_init,
      NULL, (SUBR) dssievent}
 #endif
 };
 
-LINKAGE
-
+LINKAGE_BUILTIN(dssi_localops)
