@@ -1,6 +1,7 @@
 /*
+xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
 Partikkel - a granular synthesis module for Csound 5
-Copyright (C) 2006-2008 yvind Brandtsegg, Torgeir Strand Henriksen,
+Copyright (C) 2006-2016 Oeyvind Brandtsegg, Torgeir Strand Henriksen,
 Thom Johansen
 
 This library is free software; you can redistribute it and/or
@@ -23,14 +24,13 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include <math.h>
 
 #define INITERROR(x) csound->InitError(csound, Str("partikkel: " x))
-#define PERFERROR(x) csound->PerfError(csound, Str("partikkel: " x))
+#define PERFERROR(x) csound->PerfError(csound, p->h.insdshead,Str("partikkel: " x))
 #define WARNING(x) csound->Warning(csound, Str("partikkel: " x))
 
 /* Assume csound and p pointers are always available */
 #define frand() (csound->RandMT(&p->randstate)/(double)(0xffffffff))
-
 /* linear interpolation between x and y by z
- * NOTE: arguments evaluated more than once, don't pass anything with side
+ * NOTE: arguments evaluated more than once, do not pass anything with side
  * effects
  */
 #define lrp(x, y, z) ((x) + ((y) - (x))*(z))
@@ -43,7 +43,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 /* here follows routines for maintaining a linked list of grains */
 
 /* initialises a linked list of NODEs */
-static int init_pool(GRAINPOOL *s, unsigned max_grains)
+static void init_pool(GRAINPOOL *s, unsigned max_grains)
 {
     unsigned i;
     NODE **p = &s->grainlist;
@@ -58,7 +58,6 @@ static int init_pool(GRAINPOOL *s, unsigned max_grains)
         node->next = NULL;
         p = &(node->next);
     }
-    return 1;
 }
 
 /* returns pointer to new node */
@@ -99,61 +98,61 @@ static int setup_globals(CSOUND *csound, PARTIKKEL *p)
 
     pg = csound->QueryGlobalVariable(csound, "partikkel");
     if (pg == NULL) {
-        char *t;
-        int i;
+      int i;
 
-        if (csound->CreateGlobalVariable(csound, "partikkel",
-                                         sizeof(PARTIKKEL_GLOBALS)) != 0)
-            return INITERROR("could not allocate globals");
-        pg = csound->QueryGlobalVariable(csound, "partikkel");
-        pg->rootentry = NULL;
-        /* build default tables. allocate enough for three, plus extra for the
-         * ftable data itself */
-        t = pg->tablestorage = csound->Calloc(csound, 4*sizeof(FUNC)
-                                                      + 12*sizeof(MYFLT));
-        pg->ooo_tab = (FUNC *)t;
-        t += sizeof(FUNC) + 2*sizeof(MYFLT);
-        pg->zzz_tab = (FUNC *)t;
-        t += sizeof(FUNC) + 2*sizeof(MYFLT);
-        pg->zzo_tab = (FUNC *)t;
-        t += sizeof(FUNC) + 2*sizeof(MYFLT);
-        pg->zzhhhhz_tab = (FUNC *)t;
-        /* we only fill in the entries in the FUNC struct that we use */
-        /* table with data [1.0, 1.0, 1.0], used as default by envelopes */
-        pg->ooo_tab->flen = 2;
-        pg->ooo_tab->lobits = 31;
-        for (i = 0; i <= 2; ++i)
-            pg->ooo_tab->ftable[i] = FL(1.0);
-        /* table with data [0.0, 0.0, 0.0], used as default by grain
-         * distribution table, channel masks and grain waveforms */
-        pg->zzz_tab->flen = 2;
-        pg->zzz_tab->lobits = 31;
-        /* table with data [0.0, 0.0, 1.0], used as default by gain masks,
-         * fm index table, and wave start and end freq tables */
-        pg->zzo_tab->ftable[2] = FL(1.0);
-        /* table with data [0.0, 0.0, 0.5, 0.5, 0.5, 0.5, 0.0], used as default
-         * by wave gain table */
-        for (i = 2; i <= 5; ++i)
-            pg->zzhhhhz_tab->ftable[i] = FL(0.5);
+      if (UNLIKELY(csound->CreateGlobalVariable(csound, "partikkel",
+                                                sizeof(PARTIKKEL_GLOBALS)) != 0))
+        return INITERROR("could not allocate globals");
+      pg = csound->QueryGlobalVariable(csound, "partikkel");
+      pg->rootentry = NULL;
+      /* build default tables. allocate enough for three, plus extra for the
+       * ftable data itself */
+      /* we only fill in the entries in the FUNC struct that we use */
+      /* table with data [1.0, 1.0, 1.0], used as default by envelopes */
+      pg->ooo_tab = (FUNC *)csound->Calloc(csound, sizeof(FUNC));
+      pg->ooo_tab->ftable = (MYFLT*)csound->Calloc(csound, 3*sizeof(MYFLT));
+      pg->ooo_tab->flen = 2;
+      pg->ooo_tab->lobits = 31;
+      for (i = 0; i <= 2; ++i)
+        pg->ooo_tab->ftable[i] = FL(1.0);
+      /* table with data [0.0, 0.0, 0.0], used as default by grain
+       * distribution table, channel masks and grain waveforms */
+      pg->zzz_tab = (FUNC *)csound->Calloc(csound, sizeof(FUNC));
+      pg->zzz_tab->ftable = (MYFLT*)csound->Calloc(csound, 3*sizeof(MYFLT));
+      pg->zzz_tab->flen = 2;
+      pg->zzz_tab->lobits = 31;
+      /* table with data [0.0, 0.0, 1.0], used as default by gain masks,
+       * fm index table, and wave start and end freq tables */
+      pg->zzo_tab = (FUNC *)csound->Calloc(csound, sizeof(FUNC));
+      pg->zzo_tab->ftable = (MYFLT*)csound->Calloc(csound, 4*sizeof(MYFLT));
+      pg->zzo_tab->ftable[2] = FL(1.0);
+      pg->zzo_tab->flen = 3;  /* JPff */
+      /* table with data [0.0, 0.0, 0.5, 0.5, 0.5, 0.5, 0.0], used as default
+       * by wave gain table */
+      pg->zzhhhhz_tab = (FUNC *)csound->Calloc(csound, sizeof(FUNC));
+      pg->zzhhhhz_tab->ftable = (MYFLT*)csound->Calloc(csound, 8*sizeof(MYFLT));
+      for (i = 2; i <= 5; ++i)
+        pg->zzhhhhz_tab->ftable[i] = FL(0.5);
     }
     p->globals = pg;
     if ((int)*p->opcodeid == 0) {
-        /* opcodeid 0 means we don't bother with the sync opcode */
-        p->globals_entry = NULL;
-        return OK;
+      /* opcodeid 0 means we do not bother with the sync opcode */
+      p->globals_entry = NULL;
+      return OK;
     }
     /* try to find entry corresponding to our opcodeid */
     pe = &pg->rootentry;
     while (*pe != NULL && (*pe)->id != *p->opcodeid)
-        pe = &((*pe)->next);
+      pe = &((*pe)->next);
 
     /* check if one already existed, if not, create one */
     if (*pe == NULL) {
-        *pe = csound->Malloc(csound, sizeof(PARTIKKEL_GLOBALS_ENTRY));
-        (*pe)->id = *p->opcodeid;
-        /* allocate table for sync data */
-        (*pe)->synctab = csound->Calloc(csound, 2*csound->ksmps*sizeof(MYFLT));
-        (*pe)->next = NULL;
+      *pe = csound->Malloc(csound, sizeof(PARTIKKEL_GLOBALS_ENTRY));
+      (*pe)->id = *p->opcodeid;
+      (*pe)->partikkel = p;
+      /* allocate table for sync data */
+      (*pe)->synctab = csound->Calloc(csound, 2*CS_KSMPS*sizeof(MYFLT));
+      (*pe)->next = NULL;
     }
     p->globals_entry = *pe;
     return OK;
@@ -177,7 +176,8 @@ static inline MYFLT lrplookup(FUNC *tab, unsigned phase, MYFLT zscale,
     return lrp(a, b, z);
 }
 
-static inline double intpow(MYFLT x, unsigned n)
+/* Why not use csound->intpow ? */
+static inline double intpow_(MYFLT x, unsigned n)
 {
     double ans = 1.0;
 
@@ -203,7 +203,7 @@ static inline MYFLT dsf(FUNC *tab, GRAIN *grain, double beta, MYFLT zscale,
 
     cos_beta = lrplookup(tab, fbeta, zscale, cosineshift);
     denominator = FL(1.0) - FL(2.0)*a*cos_beta + a*a;
-    if (denominator < FL(1e-4) && denominator > FL(-1e-4)) {
+    if (denominator < FL(1e-6) && denominator > FL(-1e-6)) {
         /* handle this special case to avoid divison by zero */
         result = N - FL(1.0);
     } else {
@@ -219,7 +219,7 @@ static inline MYFLT dsf(FUNC *tab, GRAIN *grain, double beta, MYFLT zscale,
 
 static int partikkel_init(CSOUND *csound, PARTIKKEL *p)
 {
-    long size;
+    uint32_t size;
     int ret;
 
     if ((ret = setup_globals(csound, p)) != OK)
@@ -263,30 +263,37 @@ static int partikkel_init(CSOUND *csound, PARTIKKEL *p)
     p->wavgaintab = *p->waveamps >= FL(0.0)
                     ? csound->FTFind(csound, p->waveamps)
                     : p->globals->zzhhhhz_tab;
+    if (*p->pantable >= FL(0.0)) {
+        p->pantab = csound->FTFind(csound, p->pantable);
+        if (!p->pantab)
+            return INITERROR("unable to load panning function table");
+    } else {
+        p->pantab = NULL; /* use default linear panning function */
+    }
 
-    if (!p->disttab)
+    if (UNLIKELY(!p->disttab))
         return INITERROR("unable to load distribution table");
-    if (!p->costab)
+    if (UNLIKELY(!p->costab))
         return INITERROR("unable to load cosine table");
-    if (!p->gainmasktab)
+    if (UNLIKELY(!p->gainmasktab))
         return INITERROR("unable to load gain mask table");
-     if (!p->channelmasktab)
+    if (UNLIKELY(!p->channelmasktab))
         return INITERROR("unable to load channel mask table");
-    if (!p->env_attack_tab || !p->env_decay_tab || !p->env2_tab)
+    if (UNLIKELY(!p->env_attack_tab || !p->env_decay_tab || !p->env2_tab))
         return INITERROR("unable to load envelope table");
-    if (!p->wavfreqstarttab)
+    if (UNLIKELY(!p->wavfreqstarttab))
         return INITERROR("unable to load start frequency scaler table");
-    if (!p->wavfreqendtab)
+    if (UNLIKELY(!p->wavfreqendtab))
         return INITERROR("unable to load end frequency scaler table");
-    if (!p->fmamptab)
+    if (UNLIKELY(!p->fmamptab))
         return INITERROR("unable to load FM index table");
-    if (!p->wavgaintab)
+    if (UNLIKELY(!p->wavgaintab))
         return INITERROR("unable to load wave gain table");
 
     p->disttabshift = sizeof(unsigned)*CHAR_BIT -
-                      (unsigned)(log((double)p->disttab->flen)/log(2.0));
+                      (unsigned)(log((double)p->disttab->flen)/log(2.0) + 0.5);
     p->cosineshift = sizeof(unsigned)*CHAR_BIT -
-                     (unsigned)(log((double)p->costab->flen)/log(2.0));
+                     (unsigned)(log((double)p->costab->flen)/log(2.0) + 0.5);
     p->zscale = FL(1.0)/FL(1 << p->cosineshift);
     p->wavfreqstartindex = p->wavfreqendindex = 0;
     p->gainmaskindex = p->channelmaskindex = 0;
@@ -294,15 +301,18 @@ static int partikkel_init(CSOUND *csound, PARTIKKEL *p)
     p->fmampindex = 0;
     p->distindex = 0;
     p->synced = 0;
+    p->graininc = 0.0;
+
     /* allocate memory for the grain mix buffer */
-    size = csound->ksmps*sizeof(MYFLT);
+    size = CS_KSMPS*sizeof(MYFLT);
     if (p->aux.auxp == NULL || p->aux.size < size)
         csound->AuxAlloc(csound, size, &p->aux);
-    memset(p->aux.auxp, 0, sizeof(MYFLT)*csound->ksmps);
+    else
+      memset(p->aux.auxp, 0, size);
 
     /* allocate memory for the grain pool and initialize it*/
-    if (*p->max_grains < FL(1.0))
-        return INITERROR("maximum number of grains need to be non-zero "
+    if (UNLIKELY(*p->max_grains < FL(1.0)))
+        return INITERROR("maximum number of grains needs to be non-zero "
                          "and positive");
     size = ((unsigned)*p->max_grains)*sizeof(NODE);
     if (p->aux2.auxp == NULL || p->aux2.size < size)
@@ -311,21 +321,20 @@ static int partikkel_init(CSOUND *csound, PARTIKKEL *p)
     init_pool(&p->gpool, (unsigned)*p->max_grains);
 
     /* find out which of the xrate parameters are arate */
-    p->grainfreq_arate = csound->GetInputArgAMask(p) & 1 ? 1 : 0;
+    p->grainfreq_arate = IS_ASIG_ARG(p->grainfreq) ? 1 : 0;
     p->out_of_voices_warning = 0; /* reset user warning indicator */
     csound->SeedRandMT(&p->randstate, NULL, csound->GetRandomSeedFromTime());
     return OK;
 }
 
-/* n is sample number for which the grain is to be scheduled */
-static int schedule_grain(CSOUND *csound, PARTIKKEL *p, NODE *node, long n)
+/* n is sample number for which the grain is to be scheduled
+ * offset is time offset for grain in seconds, passed separately for hints */
+static int schedule_grain(CSOUND *csound, PARTIKKEL *p, NODE *node, int32 n,
+                          double offset)
 {
     /* make a new grain */
     MYFLT startfreqscale, endfreqscale;
     MYFLT maskgain, maskchannel;
-    int samples;
-    double rcp_samples; /* 1/samples */
-    double phase_corr;
     GRAIN *grain = &node->grain;
     unsigned int i;
     unsigned int chan;
@@ -370,8 +379,9 @@ static int schedule_grain(CSOUND *csound, PARTIKKEL *p, NODE *node, long n)
     clip_index(p->wavgainindex, wavgains[0], wavgains[1]);
     wavgainsindex = 5*p->wavgainindex++;
 
+    graingain = *p->amplitude*maskgain;
     /* check if our mask gain is zero or if stochastic masking takes place */
-    if ((fabs(maskgain) < FL(1e-8)) || (frand() > 1.0 - *p->randommask)) {
+    if ((fabs(graingain) < FL(1e-8)) || (frand() > 1.0 - *p->randommask)) {
         /* grain is either masked out or has a zero amplitude, so we cancel it
          * and proceed with scheduling our next grain */
         return_grain(&p->gpool, node);
@@ -385,33 +395,45 @@ static int schedule_grain(CSOUND *csound, PARTIKKEL *p, NODE *node, long n)
 
     /* place a grain in between two channels according to channel mask value */
     chan = (unsigned)maskchannel;
-    if (chan >= p->num_outputs) {
+    if (UNLIKELY(chan >= p->num_outputs)) {
         return_grain(&p->gpool, node);
         return PERFERROR("channel mask specifies non-existing output channel");
     }
-    grain->gain1 = FL(1.0) - (maskchannel - chan);
-    grain->gain2 = maskchannel - chan;
+    /* use panning law table if specified */
+    if (p->pantab != NULL) {
+        unsigned tabsize = p->pantab->flen/8;
+        unsigned i1 = (unsigned)((FL(1.0) - maskchannel + 2*chan)*tabsize);
+        unsigned i2 = (unsigned)(maskchannel*tabsize);
+
+        grain->gain1 = p->pantab->ftable[i1];
+        grain->gain2 = p->pantab->ftable[i2];
+    } else {
+        grain->gain1 = FL(1.0) - (maskchannel - chan);
+        grain->gain2 = maskchannel - chan;
+    }
+
     grain->chan1 = chan;
     grain->chan2 = p->num_outputs > chan + 1 ? chan + 1 : 0;
 
-    graingain = *p->amplitude*maskgain;
     /* duration in samples */
-    samples = (int)((csound->esr*(*p->duration)/1000.0) + 0.5);
+    const double dur_samples = CS_ESR*(*p->duration)/1000.0;
     /* if grainlength is below one sample, we'll just cancel it */
-    if (samples <= 0) {
+    if (dur_samples < 1.0) {
         return_grain(&p->gpool, node);
         return OK;
     }
-    rcp_samples = 1.0/(double)samples;
-    grain->start = n;
-    grain->stop = n + samples;
-    /* if grainphase is larger than graininc, the grain is not synchronous and
-     * we'll skip sub-sample grain placement. also check for divide by zero */
-    if (p->grainphase < p->graininc && p->graininc != 0.0)
-        phase_corr = p->graininc != 0.0 ? p->grainphase/p->graininc : 0.0;
-    else
-        phase_corr = 0;
-
+    /* the grain is supposed to start at grainphase = 0, so calculate how far
+     * we overshot that and correct all relevant wave and envelope phases
+     * for proper sub-sample grain placement. if offset != 0, our grains
+     * are probably not very synchronous, and will not benefit from this.
+     * also only enable it for sufficiently high grain rates. current
+     * threshold corresponds to around 150hz */
+    const double phase_corr = offset == 0.0 && p->graininc > 0.0032
+                            ? p->grainphase/p->graininc
+                            : 0.0;
+    const double rcp_samples = 1.0/dur_samples;
+    grain->start = (unsigned)((double)n + offset*CS_ESR + phase_corr);
+    grain->stop = (unsigned)(grain->start + dur_samples - phase_corr) + 1;
     /* set up the four wavetables and dsf to use in the grain */
     for (i = 0; i < 5; ++i) {
         WAVEDATA *curwav = &grain->wav[i];
@@ -426,9 +448,8 @@ static int schedule_grain(CSOUND *csound, PARTIKKEL *p, NODE *node, long n)
         curwav->table = i != WAV_TRAINLET ? p->wavetabs[i] : p->costab;
         curwav->gain = wavgains[wavgainsindex + i + 2]*graingain;
 
-        /* check if waveform gets zero volume. if so, let's mark it as unused
-         * and move on to the next one */
-        if (fabs(curwav->gain) < FL(1e-10)) {
+        /* drop wavetables with close to zero gain */
+        if (fabs(curwav->gain) < FL(1e-8)) {
             curwav->table = NULL;
             continue;
         }
@@ -441,16 +462,17 @@ static int schedule_grain(CSOUND *csound, PARTIKKEL *p, NODE *node, long n)
             /* limit dsf harmonics to nyquist to avoid aliasing.
              * minumum number of harmonics is 2, since 1 would yield just dc,
              * which we remove anyway */
-            nh = 0.5*csound->esr/fabs(maxfreq);
+            nh = 0.5*CS_ESR/fabs(maxfreq);
             if (nh > fabs(*p->harmonics))
                 nh = fabs(*p->harmonics);
             grain->harmonics = (unsigned)nh + 1;
             if (grain->harmonics < 2)
                 grain->harmonics = 2;
             grain->falloff = *p->falloff;
-            grain->falloff_pow_N = intpow(grain->falloff, grain->harmonics);
+            grain->falloff_pow_N = intpow_(grain->falloff, grain->harmonics);
             /* normalize trainlets to uniform peak, using geometric sum */
-            if (fabs(grain->falloff) > 0.9999 && fabs(grain->falloff) < 1.0001)
+            if (FABS(grain->falloff) > FL(0.9999) &&
+                FABS(grain->falloff) < FL(1.0001))
                 /* limit case for falloff = 1 */
                 normalize = 1.0/(double)grain->harmonics;
             else
@@ -523,33 +545,31 @@ static int schedule_grain(CSOUND *csound, PARTIKKEL *p, NODE *node, long n)
 /* this function schedules the grains that are bound to happen this k-period */
 static int schedule_grains(CSOUND *csound, PARTIKKEL *p)
 {
-    long n;
+    uint32_t koffset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     NODE *node;
     MYFLT **waveformparams = &p->waveform1;
-    MYFLT grainfreq = *p->grainfreq;
+    MYFLT grainfreq = fabs(*p->grainfreq);
 
     /* krate table lookup, first look up waveform ftables */
     for (n = 0; n < 4; ++n) {
         p->wavetabs[n] = *waveformparams[n] >= FL(0.0)
                          ? csound->FTnp2Find(csound, waveformparams[n])
                          : p->globals->zzz_tab;
-        if (p->wavetabs[n] == NULL)
+        if (UNLIKELY(p->wavetabs[n] == NULL))
             return PERFERROR("unable to load waveform table");
     }
     /* look up fm envelope table for use in grains scheduled this kperiod */
     p->fmenvtab = *p->fm_env >= FL(0.0)
                   ? csound->FTFind(csound, p->fm_env)
                   : p->globals->ooo_tab;
-    if (!p->fmenvtab)
+    if (UNLIKELY(!p->fmenvtab))
         return PERFERROR("unable to load FM envelope table");
 
+    if (UNLIKELY(early)) nsmps -= early;
     /* start grain scheduling */
-    for (n = 0; n < csound->ksmps; ++n) {
-        int phase_wrapped = 0;
-        if (p->grainfreq_arate)
-            grainfreq = p->grainfreq[n];
-        p->graininc = fabs(grainfreq*csound->onedsr);
-
+    for (n = koffset; n < nsmps; ++n) {
         if (p->sync[n] >= FL(1.0)) {
             /* we got a full sync pulse, hardsync grain clock if needed */
             if (!p->synced) {
@@ -572,28 +592,35 @@ static int schedule_grains(CSOUND *csound, PARTIKKEL *p)
         }
 
         if (p->grainphase >= 1.0) {
-            /* phase has wrapped because we're at a period, so find out where
-             * to create the next grain */
+            double offset;
+
+            do
+                p->grainphase -= 1.0;
+            while (UNLIKELY(p->grainphase >= 1.0));
+            /* schedule new synchronous or synced grain */
+            /* first determine time offset for grain */
             if (*p->distribution >= FL(0.0)) {
                 /* positive distrib, choose random point in table */
                 unsigned rnd = csound->RandMT(&p->randstate);
-                MYFLT offset = p->disttab->ftable[rnd >> p->disttabshift];
-                p->nextgrainphase = *p->distribution*offset;
+                offset = p->disttab->ftable[rnd >> p->disttabshift];
+                offset *= *p->distribution;
             } else {
                 /* negative distrib, choose sequential point in table */
-                MYFLT offset = p->disttab->ftable[p->distindex++];
-                p->nextgrainphase = -*p->distribution*offset;
-                if (p->distindex >= p->disttab->flen)
+                offset = p->disttab->ftable[p->distindex++];
+                offset *= -*p->distribution;
+                if ((uint32_t)p->distindex >= p->disttab->flen)
                     p->distindex = 0;
             }
-            while (p->grainphase >= 1.0)
-                p->grainphase -= 1.0;
-            phase_wrapped = 1;
-        }
-
-        /* trigger when we have passed nextgrainphase */
-        if (p->grainphase >= p->nextgrainphase
-            && (p->prevphase < p->nextgrainphase || phase_wrapped)) {
+            /* convert offset to seconds, also limiting it to 10 seconds to
+             * avoid accidentally filling grain pool with grains which will
+             * spawn in half a day */
+            if (grainfreq < FL(0.001)) {
+                /* avoid div by zero */
+                offset = 0;
+            } else {
+                offset /= grainfreq;
+                if (offset > 10.0) offset = 10.0;
+            }
             /* check if there are any grains left in the pool */
             if (!p->gpool.free_nodes) {
                 if (!p->out_of_voices_warning) {
@@ -607,7 +634,7 @@ static int schedule_grains(CSOUND *csound, PARTIKKEL *p)
             /* check first, in case we'll change the above behaviour of
              * killing a grain */
             if (node) {
-                int ret = schedule_grain(csound, p, node, n);
+                int ret = schedule_grain(csound, p, node, n, offset);
 
                 if (ret != OK)
                     return ret;
@@ -616,15 +643,82 @@ static int schedule_grains(CSOUND *csound, PARTIKKEL *p)
             if (p->globals_entry)
                 p->globals_entry->synctab[n] = FL(1.0);
         }
+
         /* store away the scheduler phase for use in partikkelsync */
         if (p->globals_entry)
-            p->globals_entry->synctab[csound->ksmps + n] = p->grainphase;
-        p->prevphase = p->grainphase;
+            p->globals_entry->synctab[CS_KSMPS + n] = p->grainphase;
+
+        if (p->grainfreq_arate)
+            grainfreq = fabs(p->grainfreq[n]);
+        p->graininc = grainfreq*csound->onedsr;
         p->grainphase += p->graininc;
     }
     return OK;
 }
 
+/* Main synthesis loops */
+/* NOTE: the main synthesis loop is duplicated for both wavetable and
+ * trainlet synthesis for speed */
+static inline void render_wave(PARTIKKEL *p, GRAIN *grain, WAVEDATA *wav,
+                               MYFLT *buf, unsigned stop)
+{
+    unsigned n;
+    double fmenvphase = grain->envphase;
+
+    /* wavetable synthesis */
+    for (n = grain->start; n < stop; ++n) {
+        double tablen = (double)wav->table->flen;
+        unsigned x0;
+        MYFLT frac, fmenv;
+
+        /* make sure phase accumulator stays within bounds */
+        while (UNLIKELY(wav->phase >= tablen))
+            wav->phase -= tablen;
+        while (UNLIKELY(wav->phase < 0.0))
+            wav->phase += tablen;
+
+        /* sample table lookup with linear interpolation */
+        x0 = (unsigned)wav->phase;
+        frac = (MYFLT)(wav->phase - x0);
+        buf[n] += lrp(wav->table->ftable[x0], wav->table->ftable[x0 + 1],
+                      frac)*wav->gain;
+
+        fmenv = grain->fmenvtab->ftable[(size_t)(fmenvphase*FMAXLEN)
+                                        >> grain->fmenvtab->lobits];
+        fmenvphase += grain->envinc;
+        wav->phase += wav->delta + wav->delta*p->fm[n]*grain->fmamp*fmenv;
+        /* apply sweep */
+        wav->delta = wav->delta*wav->sweepdecay + wav->sweepoffset;
+     }
+}
+
+static inline void render_trainlet(PARTIKKEL *p, GRAIN *grain, WAVEDATA *wav,
+                                   MYFLT *buf, unsigned stop)
+{
+    unsigned n;
+    double fmenvphase = grain->envphase;
+
+    /* trainlet synthesis */
+    for (n = grain->start; n < stop; ++n) {
+        MYFLT fmenv;
+
+        while (UNLIKELY(wav->phase >= 1.0))
+            wav->phase -= 1.0;
+        while (UNLIKELY(wav->phase < 0.0))
+            wav->phase += 1.0;
+
+        /* dsf/trainlet synthesis */
+        buf[n] += wav->gain*dsf(p->costab, grain, wav->phase, p->zscale,
+                                p->cosineshift);
+
+        fmenv = grain->fmenvtab->ftable[(size_t)(fmenvphase*FMAXLEN)
+                                        >> grain->fmenvtab->lobits];
+        fmenvphase += grain->envinc;
+        wav->phase += wav->delta + wav->delta*p->fm[n]*grain->fmamp*fmenv;
+        wav->delta = wav->delta*wav->sweepdecay + wav->sweepoffset;
+    }
+}
+
 /* do the actual waveform synthesis */
 static inline void render_grain(CSOUND *csound, PARTIKKEL *p, GRAIN *grain)
 {
@@ -632,71 +726,23 @@ static inline void render_grain(CSOUND *csound, PARTIKKEL *p, GRAIN *grain)
     unsigned n;
     MYFLT *out1 = *(&(p->output1) + grain->chan1);
     MYFLT *out2 = *(&(p->output1) + grain->chan2);
-    unsigned stop = grain->stop > csound->ksmps
-                    ? csound->ksmps : grain->stop;
+    unsigned stop = grain->stop > CS_KSMPS
+                    ? CS_KSMPS : grain->stop;
     MYFLT *buf = (MYFLT *)p->aux.auxp;
 
+    if (grain->start >= CS_KSMPS)
+        return; /* grain starts at a later kperiod */
     for (i = 0; i < 5; ++i) {
         WAVEDATA *curwav = &grain->wav[i];
-        double fmenvphase = grain->envphase;
-        MYFLT fmenv;
 
         /* check if ftable is to be rendered */
         if (curwav->table == NULL)
             continue;
 
-        /* NOTE: the main synthesis loop is duplicated for both wavetable and
-         * trainlet synthesis for speed */
-        if (i != WAV_TRAINLET) {
-            /* wavetable synthesis */
-            for (n = grain->start; n < stop; ++n) {
-                double tablen = (double)curwav->table->flen;
-                unsigned x0;
-                MYFLT frac;
-
-                /* make sure phase accumulator stays within bounds */
-                while (curwav->phase >= tablen)
-                    curwav->phase -= tablen;
-                while (curwav->phase < 0.0)
-                    curwav->phase += tablen;
-
-                /* sample table lookup with linear interpolation */
-                x0 = (unsigned)curwav->phase;
-                frac = curwav->phase - x0;
-                buf[n] += lrp(curwav->table->ftable[x0],
-                              curwav->table->ftable[x0 + 1],
-                              frac)*curwav->gain;
-
-                fmenv = grain->fmenvtab->ftable[(unsigned)(fmenvphase*FMAXLEN)
-                                                >> grain->fmenvtab->lobits];
-                fmenvphase += grain->envinc;
-                curwav->phase += curwav->delta
-                                 + curwav->delta*p->fm[n]*grain->fmamp*fmenv;
-                /* apply sweep */
-                curwav->delta = curwav->delta*curwav->sweepdecay
-                                + curwav->sweepoffset;
-            }
-        } else {
-            /* trainlet synthesis */
-            for (n = grain->start; n < stop; ++n) {
-                while (curwav->phase >= 1.0)
-                    curwav->phase -= 1.0;
-                while (curwav->phase < 0.0)
-                    curwav->phase += 1.0;
-
-                /* dsf/trainlet synthesis */
-                buf[n] += curwav->gain*dsf(p->costab, grain, curwav->phase,
-                                           p->zscale, p->cosineshift);
-
-                fmenv = grain->fmenvtab->ftable[(unsigned)(fmenvphase*FMAXLEN)
-                                                >> grain->fmenvtab->lobits];
-                fmenvphase += grain->envinc;
-                curwav->phase += curwav->delta
-                                 + curwav->delta*p->fm[n]*grain->fmamp*fmenv;
-                curwav->delta = curwav->delta*curwav->sweepdecay
-                                + curwav->sweepoffset;
-            }
-        }
+        if (i != WAV_TRAINLET)
+            render_wave(p, grain, curwav, buf, stop);
+        else
+            render_trainlet(p, grain, curwav, buf, stop);
     }
 
     /* apply envelopes */
@@ -725,9 +771,9 @@ static inline void render_grain(CSOUND *csound, PARTIKKEL *p, GRAIN *grain)
         }
 
         /* fetch envelope values */
-        env = envtable->ftable[(unsigned)(envphase*FMAXLEN)
+        env = envtable->ftable[(size_t)(envphase*FMAXLEN)
                                 >> envtable->lobits];
-        env2 = p->env2_tab->ftable[(unsigned)(grain->envphase*FMAXLEN)
+        env2 = p->env2_tab->ftable[(size_t)(grain->envphase*FMAXLEN)
                                    >> p->env2_tab->lobits];
         env2 = FL(1.0) - grain->env2amount + grain->env2amount*env2;
         grain->envphase += grain->envinc;
@@ -744,22 +790,22 @@ static inline void render_grain(CSOUND *csound, PARTIKKEL *p, GRAIN *grain)
 
 static int partikkel(CSOUND *csound, PARTIKKEL *p)
 {
-    int ret, n;
+    int ret;
+    unsigned int n;
     NODE **nodeptr;
     MYFLT **outputs = &p->output1;
 
-    if (p->aux.auxp == NULL || p->aux2.auxp == NULL)
+    if (UNLIKELY(p->aux.auxp == NULL || p->aux2.auxp == NULL))
         return PERFERROR("not initialised");
 
-    /* schedule grains that'll happen this kperiod */
     if ((ret = schedule_grains(csound, p)) != OK)
         return ret;
 
     /* clear output buffers, we'll be accumulating our outputs */
     for (n = 0; n < p->num_outputs; ++n)
-        memset(outputs[n], 0, sizeof(MYFLT)*csound->ksmps);
+        memset(outputs[n], 0, sizeof(MYFLT)*CS_KSMPS);
 
-    /* prepare to traverse grain list, start at grain list root */
+    /* prepare to traverse grain list */
     nodeptr = &p->grainroot;
     while (*nodeptr) {
         GRAIN *grain = &((*nodeptr)->grain);
@@ -767,13 +813,16 @@ static int partikkel(CSOUND *csound, PARTIKKEL *p)
         /* render current grain to outputs */
         render_grain(csound, p, grain);
         /* check if grain is finished */
-        if (grain->stop <= csound->ksmps) {
+        if (grain->stop <= CS_KSMPS) {
             /* grain is finished, deactivate it */
             *nodeptr = return_grain(&p->gpool, *nodeptr);
         } else {
             /* extend grain lifetime with one k-period and find next grain */
-            grain->start = 0;
-            grain->stop -= csound->ksmps;
+            if (CS_KSMPS > grain->start)
+                grain->start = 0; /* grain is active */
+            else
+                grain->start -= CS_KSMPS; /* grain is not yet active */
+            grain->stop -= CS_KSMPS;
             nodeptr = &((*nodeptr)->next);
         }
     }
@@ -781,22 +830,22 @@ static int partikkel(CSOUND *csound, PARTIKKEL *p)
 }
 
 /* partikkelsync stuff */
-static int partikkelsync_init(CSOUND *csound, PARTIKKELSYNC *p)
+static int partikkelsync_init(CSOUND *csound, PARTIKKEL_SYNC *p)
 {
     PARTIKKEL_GLOBALS *pg;
     PARTIKKEL_GLOBALS_ENTRY *pe;
 
-    if ((int)*p->opcodeid == 0)
+    if (UNLIKELY((int)*p->opcodeid == 0))
         return csound->InitError(csound,
             Str("partikkelsync: opcode id needs to be a non-zero integer"));
     pg = csound->QueryGlobalVariable(csound, "partikkel");
-    if (pg == NULL || pg->rootentry == NULL)
+    if (UNLIKELY(pg == NULL || pg->rootentry == NULL))
         return csound->InitError(csound,
             Str("partikkelsync: could not find opcode id"));
     pe = pg->rootentry;
     while (pe->id != *p->opcodeid && pe->next != NULL)
         pe = pe->next;
-    if (pe->id != *p->opcodeid)
+    if (UNLIKELY(pe->id != *p->opcodeid))
         return csound->InitError(csound,
             Str("partikkelsync: could not find opcode id"));
     p->ge = pe;
@@ -805,36 +854,137 @@ static int partikkelsync_init(CSOUND *csound, PARTIKKELSYNC *p)
     return OK;
 }
 
-static int partikkelsync(CSOUND *csound, PARTIKKELSYNC *p)
+static int partikkelsync(CSOUND *csound, PARTIKKEL_SYNC *p)
 {
     /* write sync pulse data */
-    memcpy(p->syncout, p->ge->synctab, csound->ksmps*sizeof(MYFLT));
+    memcpy(p->syncout, p->ge->synctab, CS_KSMPS*sizeof(MYFLT));
     /* write scheduler phase data, if user wanted it */
     if (p->output_schedphase) {
-        memcpy(p->schedphaseout, p->ge->synctab + csound->ksmps,
-               csound->ksmps*sizeof(MYFLT));
+        memcpy(p->schedphaseout, p->ge->synctab + CS_KSMPS,
+               CS_KSMPS*sizeof(MYFLT));
     }
     /* clear first half of sync table to get rid of old sync pulses */
-    memset(p->ge->synctab, 0, csound->ksmps*sizeof(MYFLT));
+    memset(p->ge->synctab, 0, CS_KSMPS*sizeof(MYFLT));
     return OK;
 }
 
-static OENTRY localops[] = {
+static int get_global_entry(CSOUND *csound, PARTIKKEL_GLOBALS_ENTRY **entry,
+                            MYFLT opcodeid, const char *prefix)
+{
+    PARTIKKEL_GLOBALS *pg;
+    PARTIKKEL_GLOBALS_ENTRY *pe;
+
+    pg = csound->QueryGlobalVariable(csound, "partikkel");
+    if (UNLIKELY(pg == NULL))
+        return csound->InitError(csound,
+                                 Str("%s: partikkel not initialized"), prefix);
+    /* try to find entry corresponding to our opcodeid */
+    pe = pg->rootentry;
+    while (pe != NULL && pe->id != opcodeid)
+        pe = pe->next;
+
+    if (UNLIKELY(pe == NULL))
+        return csound->InitError(csound,
+                                 Str("%s: could not find opcode id"), prefix);
+    *entry = pe;
+    return OK;
+}
+
+static int partikkelget_init(CSOUND *csound, PARTIKKEL_GET *p)
+{
+    return get_global_entry(csound, &p->ge, *p->opcodeid, "partikkelget");
+}
+
+static int partikkelget(CSOUND *csound, PARTIKKEL_GET *p)
+{
+    PARTIKKEL *partikkel = p->ge->partikkel;
+
+    switch ((int)*p->index) {
+    case 0:
+        *p->valout = (MYFLT)partikkel->gainmaskindex;
+        break;
+    case 1:
+        *p->valout = (MYFLT)partikkel->wavfreqstartindex;
+        break;
+    case 2:
+        *p->valout = (MYFLT)partikkel->wavfreqendindex;
+        break;
+    case 3:
+        *p->valout = (MYFLT)partikkel->fmampindex;
+        break;
+    case 4:
+        *p->valout = (MYFLT)partikkel->channelmaskindex;
+        break;
+    case 5:
+        *p->valout = (MYFLT)partikkel->wavgainindex;
+        break;
+    }
+    return OK;
+}
+
+static int partikkelset_init(CSOUND *csound, PARTIKKEL_SET *p)
+{
+    return get_global_entry(csound, &p->ge, *p->opcodeid, "partikkelset");
+}
+
+static int partikkelset(CSOUND *csound, PARTIKKEL_SET *p)
+{
+    PARTIKKEL *partikkel = p->ge->partikkel;
+
+    switch ((int)*p->index) {
+    case 0:
+        partikkel->gainmaskindex = (unsigned)*p->value;
+        break;
+    case 1:
+        partikkel->wavfreqstartindex = (unsigned)*p->value;
+        break;
+    case 2:
+        partikkel->wavfreqendindex = (unsigned)*p->value;
+        break;
+    case 3:
+        partikkel->fmampindex = (unsigned)*p->value;
+        break;
+    case 4:
+        partikkel->channelmaskindex = (unsigned)*p->value;
+        break;
+    case 5:
+        partikkel->wavgainindex = (unsigned)*p->value;
+        break;
+    }
+    return OK;
+}
+
+static OENTRY partikkel_localops[] = {
     {
-        "partikkel", sizeof(PARTIKKEL), 5,
+        "partikkel", sizeof(PARTIKKEL), TR, 5,
         "ammmmmmm",
-        "xkiakiiikkkkikkiiaikikkkikkkkkiaaaakkkkio",
+        "xkiakiiikkkkikkiiaikikkkikkkkkiaaaakkkkioj",
         (SUBR)partikkel_init,
         (SUBR)NULL,
         (SUBR)partikkel
     },
     {
-        "partikkelsync", sizeof(PARTIKKELSYNC), 5,
+        "partikkelsync", sizeof(PARTIKKEL_SYNC), TR, 5,
         "am", "i",
         (SUBR)partikkelsync_init,
         (SUBR)NULL,
         (SUBR)partikkelsync
+    },
+    {
+        "partikkelget", sizeof(PARTIKKEL_GET), TR, 3,
+        "k", "ki",
+        (SUBR)partikkelget_init,
+        (SUBR)partikkelget,
+        (SUBR)NULL
+    },
+    {
+        "partikkelset", sizeof(PARTIKKEL_SET), TR, 3,
+        "", "kki",
+        (SUBR)partikkelset_init,
+        (SUBR)partikkelset,
+        (SUBR)NULL
     }
 };
 
-LINKAGE
+LINKAGE_BUILTIN(partikkel_localops)
+
