@@ -1,7 +1,5 @@
-/**
- * C S O U N D   V S T
- *
- * A VST plugin version of Csound, with Python scripting.
+/*
+ * C S O U N D
  *
  * L I C E N S E
  *
@@ -23,6 +21,8 @@
 #pragma warning(disable: 4786)
 #endif
 #include "CsoundFile.hpp"
+#include <boost/algorithm/string.hpp>
+#include <string.h>
 #include <ctime>
 #include <cctype>
 #include <algorithm>
@@ -31,6 +31,18 @@
 #include <sys/stat.h>
 #include <csound.h>
 
+#if defined(HAVE_MUSICXML2)
+#include "elements.h"
+#include "factory.h"
+#include "xml.h"
+#include "xmlfile.h"
+#include "xml_tree_browser.h"
+#include "xmlreader.h"
+#include "midicontextvisitor.h"
+
+using namespace MusicXML2;
+#endif
+
 void PUBLIC gatherArgs(int argc, const char **argv, std::string &commandLine)
 {
   for(int i = 0; i < argc; i++)
@@ -47,28 +59,22 @@ void PUBLIC gatherArgs(int argc, const char **argv, std::string &commandLine)
     }
 }
 
-void PUBLIC scatterArgs(const std::string buffer, std::vector<std::string> &args, std::vector<char *> &argv)
+void PUBLIC scatterArgs(const std::string buffer,
+                        std::vector<std::string> &args, std::vector<char *> &argv)
 {
-  std::string separators = " \t\n\r";
   args.clear();
-  argv.clear();
-  size_t first = 0;
-  size_t last = 0;
-  for(;;) {
-    first = buffer.find_first_not_of(separators, last);
-    if (first == std::string::npos) {
-      return;
-    }
-    last = buffer.find_first_of(separators, first);
-    if (last == std::string::npos) {
-      args.push_back(buffer.substr(first));
-      argv.push_back(const_cast<char *>(args.back().c_str()));
-      return;
-    } else {
-      args.push_back(buffer.substr(first, last - first));
-      argv.push_back(const_cast<char *>(args.back().c_str()));
-    }
+  for (int i = 0; i < argv.size(); ++i) {
+      char *arg = argv[i];
+      if (arg) {
+          free(arg);
+      }
   }
+  argv.clear();
+  boost::split(args, buffer, boost::is_any_of(" \t\n\r"), boost::token_compress_on);
+  for (int i = 0; i < args.size(); ++i) {
+      argv.push_back(const_cast<char *>(strdup(args[i].c_str())));
+  }
+  argv.push_back(0);
 }
 
 std::string PUBLIC &trim(std::string &value)
@@ -109,10 +115,13 @@ std::string PUBLIC &trimQuotes(std::string &value)
 /**
  *       Returns true if definition is a valid Csound instrument definition block.
  *       Also returns the part before the instr number, the instr number,
- *       the name (all text after the first comment on the same line as the instr number),
+ *       the name (all text after the first comment on the same line as
+ *       the instr number),
  *       and the part after the instr number, all by reference.
  */
-bool PUBLIC parseInstrument(const std::string &definition, std::string &preNumber, std::string &id, std::string &name, std::string &postNumber);
+bool PUBLIC parseInstrument(const std::string &definition, std::string &preNumber,
+                            std::string &id, std::string &name,
+                            std::string &postNumber);
 
 char staticBuffer[0x1000];
 
@@ -206,15 +215,18 @@ int CsoundFile::save(std::string filename) const
 {
   int returnValue = false;
   std::ofstream stream(filename.c_str(), std::ios::binary);
-  if(!((filename.find(".orc") == filename.npos) && (filename.find(".ORC") == filename.npos)))
+  if(!((filename.find(".orc") == filename.npos) &&
+       (filename.find(".ORC") == filename.npos)))
     {
       returnValue += exportOrchestra(stream);
     }
-  else if(!((filename.find(".sco") == filename.npos) && (filename.find(".SCO") == filename.npos)))
+  else if(!((filename.find(".sco") == filename.npos) &&
+            (filename.find(".SCO") == filename.npos)))
     {
       returnValue += exportScore(stream);
     }
-  else if(!((filename.find(".mid") == filename.npos) && (filename.find(".MID") == filename.npos)))
+  else if(!((filename.find(".mid") == filename.npos) &&
+            (filename.find(".MID") == filename.npos)))
     {
       returnValue += exportMidifile(stream);
     }
@@ -258,6 +270,59 @@ int CsoundFile::save(std::ostream &stream) const
   return returnValue;
 }
 
+#if defined(HAVE_MUSICXML2)
+
+  class CsoundFileMidiWriter : public midiwriter
+  {
+  public:
+    long tpq;
+    double tempo;
+    CsoundFileMidiWriter(CsoundFile *csoundFile_) : tpq(1000), tempo(1.0),
+                                                    csoundFile(*csoundFile_)
+    {
+      csoundFile.removeScore();
+    }
+    virtual void startPart (int instrCount)
+    {
+    }
+    virtual void newInstrument (std::string instrName, int chan=-1)
+    {
+    }
+    virtual void endPart (long date)
+    {
+    }
+    virtual void newNote (long start_, int insno_, float key_,
+                          int velocity_, int duration_)
+    {
+      double insno = double(insno_ + 1.0);
+      double start = double(start_) / double(tpq) * tempo;
+      double duration = double(duration_) / double(tpq) * tempo;
+      double key = key_;
+      double velocity = velocity_;
+      csoundFile.addNote(insno, start, duration, key, velocity);
+    }
+    virtual void tempoChange (long date, int bpm)
+    {
+      tempo = 60.0 / double(bpm);
+    }
+    virtual void pedalChange (long date, pedalType t, int value)
+    {
+    }
+    virtual void volChange (long date, int chan, int vol)
+    {
+    }
+    virtual void bankChange (long date, int chan, int bank)
+    {
+    }
+    virtual void progChange (long date, int chan, int prog)
+    {
+    }
+  protected:
+    CsoundFile &csoundFile;
+  };
+
+#endif
+
 int CsoundFile::importFile(std::string filename)
 {
   struct stat statbuffer;
@@ -267,18 +332,52 @@ int CsoundFile::importFile(std::string filename)
       return false;
     }
   std::ifstream stream(filename.c_str(), std::ios::binary);
-  if((filename.find(".orc")   != filename.npos) || (filename.find(".ORC")    != filename.npos))
+  if((filename.find(".orc")    != filename.npos) ||
+     (filename.find(".ORC")    != filename.npos))
     {
       returnValue += importOrchestra(stream);
     }
-  else if((filename.find(".sco") != filename.npos) || (filename.find(".SCO") != filename.npos))
+  else if((filename.find(".sco") != filename.npos) ||
+          (filename.find(".SCO") != filename.npos))
     {
       returnValue += importScore(stream);
     }
-  else if((filename.find(".mid") != filename.npos) || (filename.find(".MID") != filename.npos))
+  else if((filename.find(".mid") != filename.npos) ||
+          (filename.find(".MID") != filename.npos))
     {
       returnValue += importMidifile(stream);
     }
+#if defined(HAVE_MUSICXML2)
+  else if((filename.find(".xml") != filename.npos) ||
+          (filename.find(".XML") != filename.npos))
+    {
+      score.erase();
+      xmlreader xmlReader;
+      Sxmlelement sxmlElement;
+      // Try to read an SXMLFile out of the MusicXML file.
+      SXMLFile sxmlFile = xmlReader.read(filename.c_str());
+      if (sxmlFile) {
+        // Get the document tree of XML elements from the SXMLFile.
+        sxmlElement = sxmlFile->elements();
+      }
+      if (sxmlElement) {
+        // Create a CsoundFileMidiWriter that is attached to this Score.
+        CsoundFileMidiWriter csoundFileMidiWriter(this);
+        // Create a midicontextvisitor, which calls into an abstract
+        // midiwriter interface, which is attached to our
+        // CsoundFileMidiWriter, which implements that midiwriter
+        // interface.
+        midicontextvisitor midicontextvisitor_(csoundFileMidiWriter.tpq,
+                                               &csoundFileMidiWriter);
+        // Create an xml_tree_browser that is attached to our midicontextvisitor.
+        xml_tree_browser xmlTreeBrowser(&midicontextvisitor_);
+        // The xml_tree_browser will carry the midicontextvisitor to
+        // all the elements of the document tree, in the proper order,
+        // calling newNote as appropriate.
+        xmlTreeBrowser.browse(*sxmlElement);
+      }
+    }
+#endif
   else
     {
       returnValue += importFile(stream);
@@ -425,7 +524,8 @@ void CsoundFile::removeCommand()
 std::string CsoundFile::getOrcFilename() const
 {
   std::string buffer;
-  scatterArgs(command, const_cast< std::vector<std::string> & >(args), const_cast< std::vector<char *> &>(argv));
+  scatterArgs(command, const_cast< std::vector<std::string> & >(args),
+              const_cast< std::vector<char *> &>(argv));
   if(args.size() >= 3)
     {
       buffer = args[args.size() - 2];
@@ -436,7 +536,8 @@ std::string CsoundFile::getOrcFilename() const
 std::string CsoundFile::getScoFilename() const
 {
   std::string buffer;
-  scatterArgs(command, const_cast< std::vector<std::string> & >(args), const_cast< std::vector<char *> &>(argv));
+  scatterArgs(command, const_cast< std::vector<std::string> & >(args),
+              const_cast< std::vector<char *> &>(argv));
   if(args.size() >= 3)
     {
       buffer = args[args.size() - 1];
@@ -447,7 +548,8 @@ std::string CsoundFile::getScoFilename() const
 std::string CsoundFile::getMidiFilename() const
 {
   std::string buffer;
-  scatterArgs(command, const_cast< std::vector<std::string> & >(args), const_cast< std::vector<char *> &>(argv));
+  scatterArgs(command, const_cast< std::vector<std::string> & >(args),
+              const_cast< std::vector<char *> &>(argv));
   for(int i = 1, n = args.size() - 2; i < n; i++)
     {
       std::string buffer = args[i];
@@ -581,45 +683,60 @@ void CsoundFile::removeMidifile()
   midifile.resize(0);
 }
 
-void CsoundFile::addNote(double p1, double p2, double p3, double p4, double p5, double p6, double p7, double p8, double p9, double p10, double p11)
+void CsoundFile::addNote(double p1, double p2, double p3, double p4,
+                         double p5, double p6, double p7, double p8,
+                         double p9, double p10, double p11)
 {
   char note[0x100];
-  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11);
+  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g"
+          " %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11);
   addScoreLine(note);
 }
 
-void CsoundFile::addNote(double p1, double p2, double p3, double p4, double p5, double p6, double p7, double p8, double p9, double p10)
+void CsoundFile::addNote(double p1, double p2, double p3, double p4,
+                         double p5, double p6, double p7, double p8,
+                         double p9, double p10)
 {
   char note[0x100];
-  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8, p9, p10);
+  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g"
+          " %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8, p9, p10);
   addScoreLine(note);
 }
 
-void CsoundFile::addNote(double p1, double p2, double p3, double p4, double p5, double p6, double p7, double p8, double p9)
+void CsoundFile::addNote(double p1, double p2, double p3, double p4,
+                         double p5, double p6, double p7, double p8,
+                         double p9)
 {
   char note[0x100];
-  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8, p9);
+  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g",
+          p1, p2, p3, p4, p5, p6, p7, p8, p9);
   addScoreLine(note);
 }
 
-void CsoundFile::addNote(double p1, double p2, double p3, double p4, double p5, double p6, double p7, double p8)
+void CsoundFile::addNote(double p1, double p2, double p3, double p4,
+                         double p5, double p6, double p7, double p8)
 {
   char note[0x100];
-  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8);
+  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g",
+          p1, p2, p3, p4, p5, p6, p7, p8);
   addScoreLine(note);
 }
 
-void CsoundFile::addNote(double p1, double p2, double p3, double p4, double p5, double p6, double p7)
+void CsoundFile::addNote(double p1, double p2, double p3, double p4,
+                         double p5, double p6, double p7)
 {
   char note[0x100];
-  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7);
+  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g",
+          p1, p2, p3, p4, p5, p6, p7);
   addScoreLine(note);
 }
 
-void CsoundFile::addNote(double p1, double p2, double p3, double p4, double p5, double p6)
+void CsoundFile::addNote(double p1, double p2, double p3, double p4,
+                         double p5, double p6)
 {
   char note[0x100];
-  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6);
+  sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g",
+          p1, p2, p3, p4, p5, p6);
   addScoreLine(note);
 }
 
@@ -700,7 +817,8 @@ int CsoundFile::getInstrumentCount() const
           return index;
         }
       endDefinition += 6;
-      std::string definition = orchestra.substr(beginDefinition, endDefinition - beginDefinition);
+      std::string definition = orchestra.substr(beginDefinition,
+                                                endDefinition - beginDefinition);
       std::string pre;
       std::string id;
       std::string name;
@@ -735,7 +853,8 @@ bool CsoundFile::getInstrument(int number, std::string &definition_) const
           return false;
         }
       endDefinition += 6;
-      std::string definition = orchestra.substr(beginDefinition, endDefinition - beginDefinition);
+      std::string definition = orchestra.substr(beginDefinition,
+                                                endDefinition - beginDefinition);
       std::string pre;
       std::string id;
       std::string name;
@@ -753,6 +872,44 @@ bool CsoundFile::getInstrument(int number, std::string &definition_) const
   return false;
 }
 
+std::string CsoundFile::getInstrument(std::string name) const
+{
+  std::string definition;
+  (void) getInstrument(name, definition);
+  return definition;
+}
+
+std::string CsoundFile::getInstrument(int number) const
+{
+  std::string definition;
+  (void) getInstrument(number, definition);
+  return definition;
+}
+
+std::string CsoundFile::getInstrumentBody(std::string name) const
+{
+  std::string definition;
+  getInstrument(name, definition);
+  std::string pre;
+  std::string id;
+  std::string name_;
+  std::string post;
+  (void) parseInstrument(definition, pre, id, name_, post);
+  return definition;
+}
+
+std::string CsoundFile::getInstrumentBody(int number) const
+{
+  std::string definition;
+  getInstrument(number, definition);
+  std::string pre;
+  std::string id;
+  std::string name;
+  std::string post;
+  (void) parseInstrument(definition, pre, id, name, post);
+  return post;
+}
+
 std::map<int, std::string> CsoundFile::getInstrumentNames() const
 {
   std::map<int, std::string> instrumentNames;
@@ -771,7 +928,8 @@ std::map<int, std::string> CsoundFile::getInstrumentNames() const
           return instrumentNames;
         }
       endDefinition += 6;
-      std::string definition = orchestra.substr(beginDefinition, endDefinition - beginDefinition);
+      std::string definition = orchestra.substr(beginDefinition,
+                                                endDefinition - beginDefinition);
       std::string pre;
       std::string id;
       std::string name;
@@ -801,7 +959,8 @@ std::map<int, std::string> CsoundFile::getInstrumentNames() const
 //                      return false;
 //              }
 //              endDefinition += 6;
-//              std::string definition = orchestra.substr(beginDefinition, endDefinition - beginDefinition);
+//              std::string definition = orchestra.substr(beginDefinition,
+//                                                 endDefinition - beginDefinition);
 //              std::string pre;
 //              std::string id;
 //              std::string name;
@@ -838,7 +997,8 @@ bool CsoundFile::getInstrument(std::string name_, std::string &definition_) cons
           return false;
         }
       endDefinition += 6;
-      std::string definition = orchestra.substr(beginDefinition, endDefinition - beginDefinition);
+      std::string definition = orchestra.substr(beginDefinition,
+                                                endDefinition - beginDefinition);
       std::string pre;
       std::string id;
       std::string name;
@@ -856,6 +1016,42 @@ bool CsoundFile::getInstrument(std::string name_, std::string &definition_) cons
   return false;
 }
 
+double CsoundFile::getInstrumentNumber(std::string name_) const
+{
+  trim(name_);
+  int beginDefinition = 0;
+  int endDefinition = 0;
+  for(;;)
+    {
+      beginDefinition = findToken(orchestra, "instr", beginDefinition);
+      if(beginDefinition == -1)
+        {
+          return false;
+        }
+      endDefinition = findToken(orchestra, "endin", beginDefinition);
+      if(endDefinition == -1)
+        {
+          return false;
+        }
+      endDefinition += 6;
+      std::string definition = orchestra.substr(beginDefinition,
+                                                endDefinition - beginDefinition);
+      std::string pre;
+      std::string id;
+      std::string name;
+      std::string post;
+      if(parseInstrument(definition, pre, id, name, post))
+        {
+          if(name_.compare(name) == 0 || id.compare(name) == 0)
+            {
+              return atof(id.c_str());
+            }
+        }
+      beginDefinition++;
+    }
+  return -1.0;
+}
+
 bool CsoundFile::exportForPerformance() const
 {
   std::string orcFilename = getOrcFilename();
@@ -930,7 +1126,9 @@ int CsoundFile::importArrangement(std::istream &stream)
 
 int CsoundFile::exportArrangement(std::ostream &stream) const
 {
-  for(std::vector<std::string>::const_iterator it = arrangement.begin(); it != arrangement.end(); ++it)
+  for(std::vector<std::string>::const_iterator it = arrangement.begin();
+      it != arrangement.end();
+      ++it)
     {
       stream << (*it).c_str() << std::endl;
     }
@@ -976,9 +1174,11 @@ int CsoundFile::exportArrangementForPerformance(std::ostream &stream) const
               std::string preNumber;
               std::string id;
               std::string postNumber;
-              if(parseInstrument(definition, preNumber, id, instrumentName, postNumber))
+              if(parseInstrument(definition, preNumber, id,
+                                 instrumentName, postNumber))
                 {
-                  stream << std::endl << "instr " << (i + 1) << " ; " << instrumentName << std::endl << postNumber << std::endl;
+                  stream << std::endl << "instr " << (i + 1) <<
+                    " ; " << instrumentName << std::endl << postNumber << std::endl;
                   stream.flush();
                 }
             }
