@@ -52,13 +52,13 @@ typedef float mus_sample_t;
 
 /********** ANALYSIS PARAMETERS ***********/
 /* start time */
-#define  ATSA_START 0.0
+#define  ATSA_START 0.0f
 /* duration */
-#define  ATSA_DUR 0.0
+#define  ATSA_DUR 0.0f
 /* lowest frequency (hertz)  */
-#define  ATSA_LFREQ 20.0
+#define  ATSA_LFREQ 20.0f
 /* highest frequency (hertz) */
-#define  ATSA_HFREQ 20000.0
+#define  ATSA_HFREQ 20000.0f
 /* frequency deviation (ratio) */
 #define  ATSA_FREQDEV 0.1f
 /* number of f0 cycles in window */
@@ -68,23 +68,23 @@ typedef float mus_sample_t;
 /* window size */
 #define  ATSA_WSIZE 1024
 /* hop size proportional to window size (ratio) */
-#define  ATSA_HSIZE 0.25
+#define  ATSA_HSIZE 0.25f
 /* lowest magnitude for peaks (amp) */
-#define  ATSA_LMAG  -60.0
+#define  ATSA_LMAG  -60.0f
 /* length of analysis tracks (frames) */
 #define  ATSA_TRKLEN 3
 /* minimum short partial length (frames) */
 #define  ATSA_MSEGLEN 3
 /* minimum short partial SMR avreage (dB SPL) */
-#define  ATSA_MSEGSMR 60.0
+#define  ATSA_MSEGSMR 60.0f
 /* minimum gap length (frames) */
 #define  ATSA_MGAPLEN 3
 /* threshold for partial SMR average (dB SPL) */
-#define  ATSA_SMRTHRES 30.0
+#define  ATSA_SMRTHRES 30.0f
 /* last peak contribution for tracking (ratio) */
-#define  ATSA_LPKCONT 0.0
+#define  ATSA_LPKCONT 0.0f
 /* SMR contribution for tracking (ratio) */
-#define  ATSA_SMRCONT 0.5
+#define  ATSA_SMRCONT 0.5f
 /* minimum number of frames for analysis (frames) */
 #define ATSA_MFRAMES 4
 /* default analysis file type
@@ -98,7 +98,7 @@ typedef float mus_sample_t;
 #if defined(LINUX) || defined(MACOSX)
 #  define ATSA_RES_FILE "/tmp/atsa_res.wav"
 #else
-#  define ATSA_RES_FILE "atsa_res.wav"
+#  define ATSA_RES_FILE "/atsa_res.wav"
 #endif
 
 /* constants and macros */
@@ -117,8 +117,8 @@ typedef float mus_sample_t;
                                   3150.0, 3700.0, 4400.0, 5300.0, 6400.0,   \
                                   7700.0, 9500.0, 12000.0, 15500.0, 20000.0}
 
-#define  AMP_DB(amp)  ((amp) != 0.0 ? (float) log10((amp) * 20.0) : -32767.0f)
-#define  DB_AMP(db)   ((float) pow(10.0, (db) / 20.0))
+//#define  AMP_DB(amp)  ((amp) != 0.0 ? (float) log10((amp) * 20.0) : -32767.0f)
+//#define  DB_AMP(db)   ((float) pow(10.0, (db) / 20.0))
 
 /* data structures */
 
@@ -533,14 +533,14 @@ static int main_anal(CSOUND *csound, char *soundfile, char *ats_outfile,
     /* open output file */
     fd = csound->FileOpen2(csound, &outfile, CSFILE_STD, ats_outfile, "wb",
                           NULL, CSFTYPE_ATS, 0);
-    if (fd == NULL) {
-      csound->Die(csound, Str("\n Could not open %s for writing, bye...\n"),
+    if (UNLIKELY(fd == NULL)) {
+      csound->Die(csound, Str("\nCould not open %s for writing, bye...\n"),
                   ats_outfile);
     }
     /* call tracker */
     sound = tracker(csound, anargs, soundfile, resfile);
     /* save sound */
-    if (sound != NULL) {
+    if (LIKELY(sound != NULL)) {
       csound->Message(csound, Str("saving ATS data..."));
       ats_save(csound, sound, outfile, anargs->SMR_thres, anargs->type);
                       csound->Message(csound, Str("done!\n"));
@@ -721,7 +721,20 @@ static int atsa_main(CSOUND *csound, int argc, char **argv)
         soundfile == NULL || soundfile[0] == '\0' ||
         ats_outfile == NULL || ats_outfile[0] == '\0')
       usage(csound);
+#ifdef WIN32
+    {
+      char buffer[160];
+      char * tmp = getenv("TEMP");
+      strncpy(buffer, tmp, 160);
+      // MKG 2014 Jan 29: No linkage for strlcat with MinGW here.
+      // but wrong; corrected
+      //strlcat(buffer, ATSA_RES_FILE, 160);
+      strncat(buffer, ATSA_RES_FILE, 160-strlen(buffer)); buffer[159] = '\0';
+      val = main_anal(csound, soundfile, ats_outfile, anargs, buffer);
+    }
+#else
     val = main_anal(csound, soundfile, ats_outfile, anargs, ATSA_RES_FILE);
+#endif
     csound->Free(csound, anargs);
     return (val);
 }
@@ -751,7 +764,7 @@ static double frq2bark(double frq, double *edges)
     band = find_band(frq, edges);
     lo_frq = edges[band];
     hi_frq = edges[band + 1];
-    return (1 + band + fabs(log10(frq / lo_frq) / log10(lo_frq / hi_frq)));
+    return (1.0 + band + fabs(log10(frq / lo_frq) / log10(lo_frq / hi_frq)));
 }
 
 /* find_band
@@ -1372,7 +1385,7 @@ static ATS_PEAK *update_tracks(CSOUND *csound, ATS_PEAK *tracks,
 static int residual_get_N(int M, int min_fft_size, int factor);
 static void residual_get_bands(double fft_mag, double *true_bands,
                                int *limits, int bands);
-static double residual_compute_time_domain_energy(ATS_FFT *fft_struct);
+//static double residual_compute_time_domain_energy(ATS_FFT *fft_struct);
 static double residual_get_band_energy(int lo, int hi, ATS_FFT *fft_struct,
                                        double norm);
 static void residual_compute_band_energy(ATS_FFT *fft_struct,
@@ -1397,25 +1410,25 @@ static void residual_get_bands(double fft_mag, double *true_bands,
       limits[k] = (int)floor(true_bands[k] / fft_mag);
 }
 
-static double residual_compute_time_domain_energy(ATS_FFT *fft)
-{
-    /* Parseval's Theorem states:
+/* static double residual_compute_time_domain_energy(ATS_FFT *fft) */
+/* { */
+/*     /\* Parseval's Theorem states: */
 
-       N-1                   N-1
-       sum(|x(n)^2|) =  1/N* sum (|X(k)|^2)
-       n=0                   k=0
+/*        N-1                   N-1 */
+/*        sum(|x(n)^2|) =  1/N* sum (|X(k)|^2) */
+/*        n=0                   k=0 */
 
-       then we multiply the time domain energy by 1/2
-       because we only compute frequency energy between
-       0 Hz and Nyquist only (0 -> N/2)
-     */
-    int     n;
-    double  sum = 0.0;
+/*        then we multiply the time domain energy by 1/2 */
+/*        because we only compute frequency energy between */
+/*        0 Hz and Nyquist only (0 -> N/2) */
+/*      *\/ */
+/*     int     n; */
+/*     double  sum = 0.0; */
 
-    for (n = 0; n < fft->size; n++)
-      sum += fabs((double) fft->data[n] * (double) fft->data[n]);
-    return (sum);
-}
+/*     for (n = 0; n < fft->size; n++) */
+/*       sum += fabs((double) fft->data[n] * (double) fft->data[n]); */
+/*     return (sum); */
+/* } */
 
 static double residual_get_band_energy(int lo, int hi, ATS_FFT *fft,
                                        double norm)
@@ -1465,9 +1478,9 @@ static void residual_compute_band_energy(ATS_FFT *fft, int *band_limits,
 static void residual_analysis(CSOUND *csound, char *file, ATS_SOUND *sound)
 {
     int     file_sampling_rate, sflen, hop, M, N, frames, *band_limits;
-    int     smp = 0, M_2, st_pt, filptr, i, frame_n, k;
+    int     M_2, st_pt, filptr, i, frame_n, k;
     double  norm = 1.0, threshold, fft_mag, **band_arr = NULL, *band_energy;
-    double  time_domain_energy = 0.0, freq_domain_energy = 0.0, sum = 0.0;
+    //double  time_domain_energy = 0.0, freq_domain_energy = 0.0, sum = 0.0;
     double  edges[ATSA_CRITICAL_BANDS + 1] = ATSA_CRITICAL_BAND_EDGES;
     ATS_FFT fft;
     SF_INFO sfinfo;
@@ -1499,7 +1512,7 @@ static void residual_analysis(CSOUND *csound, char *file, ATS_SOUND *sound)
     fft.size = N;
     fft.rate = file_sampling_rate;
     fft.data = (MYFLT *) csound->Malloc(csound, (N + 2) * sizeof(MYFLT));
-    threshold = AMP_DB(ATSA_NOISE_THRESHOLD);
+    threshold = /*AMP_DB*/(ATSA_NOISE_THRESHOLD);
     frames = sound->frames;
     fft_mag = (double) file_sampling_rate / (double) N;
     band_limits =
@@ -1524,17 +1537,17 @@ static void residual_analysis(CSOUND *csound, char *file, ATS_SOUND *sound)
           fft.data[(k + st_pt) % N] = (MYFLT) bufs[0][filptr];
         filptr++;
       }
-      smp = filptr - M_2 - 1;
-      time_domain_energy = residual_compute_time_domain_energy(&fft);
+      //smp = filptr - M_2 - 1;
+      //time_domain_energy = residual_compute_time_domain_energy(&fft);
       /* take the fft */
       csound->RealFFTnp2(csound, fft.data, N);
       residual_compute_band_energy(&fft, band_limits, ATSA_CRITICAL_BANDS + 1,
                                    band_energy, norm);
-      sum = 0.0;
-      for (k = 0; k < ATSA_CRITICAL_BANDS; k++) {
-        sum += band_energy[k];
-      }
-      freq_domain_energy = 2.0 * sum;
+      //sum = 0.0;
+      //for (k = 0; k < ATSA_CRITICAL_BANDS; k++) {
+      //  sum += band_energy[k];
+      //}
+      //freq_domain_energy = 2.0 * sum;
       for (k = 0; k < ATSA_CRITICAL_BANDS; k++) {
         if (band_energy[k] < threshold) {
           band_arr[k][frame_n] = 0.0;
@@ -1932,11 +1945,13 @@ static void ats_save(CSOUND *csound, ATS_SOUND *sound, FILE *outfile,
     header.typ = (double) type;
     /* write header */
     fseek(outfile, 0, SEEK_SET);
-    fwrite(&header, 1, sizeof(ATS_HEADER), outfile);
+    if (UNLIKELY(1!=fwrite(&header, sizeof(ATS_HEADER), 1, outfile)))
+      fprintf(stderr, Str("Write failure\n"));
     /* write frame data */
     for (frm = 0; frm < sound->frames; frm++) {
       daux = sound->time[0][frm];
-      fwrite(&daux, 1, sizeof(double), outfile);
+      if (UNLIKELY(1!=fwrite(&daux, sizeof(double), 1, outfile)))
+        fprintf(stderr, Str("Write failure\n"));
       for (i = 0; i < sound->partials; i++) {
         /* we ouput data in increasing frequency order
          * and we check for dead partials
@@ -1946,12 +1961,15 @@ static void ats_save(CSOUND *csound, ATS_SOUND *sound, FILE *outfile,
           par = sound->av[i].track;
           /* output data to file */
           daux = sound->amp[par][frm];
-          fwrite(&daux, 1, sizeof(double), outfile);
+          if (UNLIKELY(1!=fwrite(&daux, sizeof(double), 1, outfile)))
+            fprintf(stderr, Str("Write failure\n"));
           daux = sound->frq[par][frm];
-          fwrite(&daux, 1, sizeof(double), outfile);
+          if (UNLIKELY(1!=fwrite(&daux, sizeof(double), 1, outfile)))
+            fprintf(stderr, Str("Write failure\n"));
           if (type == 2 || type == 4) {
             daux = sound->pha[par][frm];
-            fwrite(&daux, 1, sizeof(double), outfile);
+            if (UNLIKELY(1!=fwrite(&daux, sizeof(double), 1, outfile)))
+              fprintf(stderr, Str("Write failure\n"));
           }
         }
       }
@@ -1959,7 +1977,8 @@ static void ats_save(CSOUND *csound, ATS_SOUND *sound, FILE *outfile,
       if (type == 3 || type == 4) {
         for (i = 0; i < ATSA_CRITICAL_BANDS; i++) {
           daux = sound->band_energy[i][frm];
-          fwrite(&daux, 1, sizeof(double), outfile);
+          if (UNLIKELY(1!=fwrite(&daux, sizeof(double), 1, outfile)))
+            fprintf(stderr, Str("Write failure\n"));
         }
       }
     }
@@ -2144,7 +2163,7 @@ static ATS_SOUND *tracker(CSOUND *csound, ANARGS *anargs, char *soundfile,
       i_tmp =
           (ATSA_MGAPLEN < anargs->frames) ? ATSA_MGAPLEN : anargs->frames - 1;
       csound->Warning(csound,
-                      Str("min. gap length %d out of bounds, " "forced to: %d"),
+                      Str("min. gap length %d out of bounds, forced to: %d"),
                       anargs->min_gap_len, i_tmp);
       anargs->min_gap_len = i_tmp;
     }
@@ -2363,8 +2382,20 @@ static ATS_SOUND *tracker(CSOUND *csound, ANARGS *anargs, char *soundfile,
     csound->Free(csound, bufs);
     /* analyse residual */
     if (anargs->type == 3 || anargs->type == 4) {
+#ifdef WIN32
+      char buffer[160];
+      char * tmp = getenv("TEMP");
+      strncpy(buffer, tmp, 160);
+      // MKG 2014 Jan 29: No linkage for strlcat with MinGW here.
+      // snd corrected
+      //strlcat(buffer, ATSA_RES_FILE, 160);
+      strncat(buffer, ATSA_RES_FILE, 159-strlen(buffer)); buffer[159]='\0';
+      csound->Message(csound, Str("Analysing residual..."));
+      residual_analysis(csound, buffer, sound);
+#else
       csound->Message(csound, Str("Analysing residual..."));
       residual_analysis(csound, ATSA_RES_FILE, sound);
+#endif
       csound->Message(csound, Str("done!\n"));
     }
     csound->Message(csound, Str("tracking completed.\n"));
@@ -2810,7 +2841,7 @@ int atsa_init_(CSOUND *csound)
     if (!retval) {
       retval =
           csound->SetUtilityDescription(csound, "atsa",
-                                        "Soundfile analysis for ATS opcodes");
+                                        Str("Soundfile analysis for ATS opcodes"));
     }
     return retval;
 }
